# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc
import base64
from datetime import datetime
from datetime import timedelta

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

from barbican_tempest_plugin.tests.rbac.v1 import base as rbac_base


CONF = config.CONF


class BarbicanV1RbacSecrets:

    @abc.abstractmethod
    def test_create_secret(self):
        """Test add_secret policy.

        Testing: POST /v1/secrets
        This test must check:
          * whether the persona can create an empty secret
          * whether the persona can create a secret with a symmetric key
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_list_secrets(self):
        """Test get_secrets policy.

        Testing: GET /v1/secrets
        This test must check:
          * whether the persona can list secrets within their project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_secret(self):
        """Test deleting a secret.

        Testing: DEL /v1/secrets/{secret_id}
        This test must check:
          * whether the persona can delete a secret in their project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_secret(self):
        """Test get_secret policy.

        Testing: GET /v1/secrets/{secret_id}
        This test must check:
          * whether the persona can get a specific secret within their project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_secret_payload(self):
        """Test get_secret payload policy.

        Testing: GET /v1/secrets/{secret_id}/payload
        This test must check:
          * whether the persona can get a secret payload
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_put_secret_payload(self):
        """Test put_secret policy.

        Testing: PUT /v1/secrets/{secret_id}
        This test must check:
          * whether the persona can add a paylod to an empty secret
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_other_project_secret(self):
        """Test get_secrets policy

        Testing: GET /v1/secrets/{secret_id}
        This test must check:
          * whether the persona can get secret metadata for a secret that
            belongs to a different project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_other_project_secret_payload(self):
        """Test get_secrets policy

        Testing: GET /v1/secrets/{secret_id}/payload
        This test must check:
          * whether the persona can get secret payload for a secret that
            belongs to a different project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_put_other_project_secret_payload(self):
        """Test put_secret policy.

        Testing: PUT /v1/secrets/{secret_id}
        This test must check:
          * whether the persona can PUT the secret payload in a 2-step
            create when the first step is done by a member of a different
            project.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_other_project_secret(self):
        """Test delete_secret policy.

        Testing: DELETE /v1/secrets/{secret_id}
        This test must check:
          * whether the persona can delete a secret that belongs to a
            different project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_secret_acl(self):
        """Test GET /v1/secrets/{secret_id}/acl policy

        This test must check:
          * whether the persona can get the ACL for a secret
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_put_secret_acl(self):
        """Test PUT /v1/secrets/{secret_id}/acl policy

        This test must check:
          * whether the persona can overwrite the ACL for a secret
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_patch_secret_acl(self):
        """Test PATCH /v1/secrets/{secret_id}/acl policy

        This test must check:
          * whether the persona can modify the ACL for a secret
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_secret_acl(self):
        """Test DELETE /v1/secrets/{secret_id}/acl policy

        This test must check:
          * whether the persona can delete the ACL for a secret
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_other_secret_acl(self):
        """Test GET /v1/secrets/{secret_id}/acl policy

        This test must check:
          * whether the persona can get the ACL for a secret
            that belongs to a different project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_put_other_secret_acl(self):
        """Test PUT /v1/secrets/{secret_id}/acl policy

        This test must check:
          * whether the persona can overwrite the ACL for a secret
            that belongs to a different project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_patch_other_secret_acl(self):
        """Test PATCH /v1/secrets/{secret_id}/acl policy

        This test must check:
          * whether the persona can modify the ACL for a secret
            that belongs to a different project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_other_secret_acl(self):
        """Test DELETE /v1/secrets/{secret_id}/acl policy

        This test must check:
          * whether the persona can delete the ACL for a secret
            that belongs to a different project
        """
        raise NotImplementedError


class BarbicanV1_1SecretConsumers:

    @abc.abstractmethod
    def test_list_secret_consumers(self):
        """Test list_secret_consumers policy

        Testing: GET /v1/secrets/{secret-id}/consumers
        This test must check:
          * whether the persona can list a secrets consumers
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_create_secret_consumer(self):
        """Test create_secret_consumer policy

        Testing: POST /v1/secrets/{secret-id}/consumers
        This test must check:
          * whether the persona can create a consumer of the secret
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_secret_consumer(self):
        """Test delete_secret_consumer policy

        Testing: DELETE /v1/secrets/{secret-id}/consumers
        This test must check:
          * whether the persona can delete a consumer of the secret
        """
        raise NotImplementedError


class ProjectReaderBase(rbac_base.BarbicanV1RbacBase):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.secret_v1.SecretClient()

    def setUp(self):
        super().setUp()
        self.secret_id = self.create_test_secret(
            self.secret_client,
            data_utils.rand_name('test-secrets'),
            'THIS_IS_A_SECRET_PASSPHRASE')
        self.other_secret_id = self.create_test_secret(
            self.other_secret_client,
            data_utils.rand_name('test-secrets'),
            'THIS_IS_SOMEONE_ELSES_SECRET_PASSPHRASE')
        self.valid_acl = {
            "read": {
                "users": [self.other_secret_client.user_id],
                "project-access": True
            }
        }


class ProjectReaderTests(ProjectReaderBase, BarbicanV1RbacSecrets):

    @decorators.idempotent_id('e4dfbae6-faca-42a7-a06b-2655e29df193')
    def test_create_secret(self):
        """Test add_secret policy."""
        self.assertRaises(exceptions.Forbidden, self.client.create_secret)

        key = rbac_base.create_aes_key()
        expire_time = (datetime.utcnow() + timedelta(days=5))

        self.assertRaises(
            exceptions.Forbidden, self.client.create_secret,
            expiration=expire_time.isoformat(), algorithm="aes",
            bit_length=256, mode="cbc", payload=key,
            payload_content_type="application/octet-stream",
            payload_content_encoding="base64"
        )

    @decorators.idempotent_id('f2649794-10d2-4742-a81c-af78eb3d9c0e')
    def test_list_secrets(self):
        """Test get_secrets policy."""
        # create two secrets
        self.create_empty_secret_admin('secret_1')
        self.create_empty_secret_admin('secret_2')

        # list secrets with name secret_1
        self.assertRaises(
            exceptions.Forbidden,
            self.client.list_secrets,
            name='secret_1'
        )

        # list secrets with name secret_2
        self.assertRaises(
            exceptions.Forbidden,
            self.client.list_secrets,
            name='secret_2'
        )

        # list all secrets
        self.assertRaises(exceptions.Forbidden, self.client.list_secrets)

    @decorators.idempotent_id('6a4cfca5-1841-49f4-ae1d-bbde0fa94bd7')
    def test_delete_secret(self):
        """Test delete_secrets policy."""
        sec = self.create_empty_secret_admin('secret_1')
        uuid = self.ref_to_uuid(sec['secret_ref'])

        self.assertRaises(
            exceptions.Forbidden,
            self.client.delete_secret,
            secret_id=uuid
        )

    @decorators.idempotent_id('9c5b46b4-8f0b-4f75-b751-61ddf943fbf3')
    def test_get_secret(self):
        """Test get_secret policy."""
        sec = self.create_empty_secret_admin('secret_1')
        uuid = self.ref_to_uuid(sec['secret_ref'])
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_secret_metadata,
            secret_id=uuid
        )

    @decorators.idempotent_id('b2760216-e492-4081-b981-a5d40bcc6a0e')
    def test_get_secret_payload(self):
        """Test get_secret payload policy."""
        key, sec = self.create_aes_secret_admin('secret_1')
        uuid = self.ref_to_uuid(sec['secret_ref'])

        # Retrieve the payload
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_secret_payload,
            secret_id=uuid
        )

    @decorators.idempotent_id('64b4e2e7-0121-46e7-949f-34332efdec6f')
    def test_put_secret_payload(self):
        """Test put_secret policy."""
        sec = self.create_empty_secret_admin('secret_1')
        uuid = self.ref_to_uuid(sec['secret_ref'])

        key = rbac_base.create_aes_key()

        # Associate the payload with the created secret
        self.assertRaises(
            exceptions.Forbidden,
            self.client.put_secret_payload,
            secret_id=uuid, payload=key
        )

    @decorators.idempotent_id('5219c830-fe82-4f3b-9eda-e3b5e918ba60')
    def test_get_other_project_secret(self):
        other_secret_id = self.create_other_project_secret(
            'get_other_secret',
            payload='¡Muy secreto!')
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_secret_metadata,
            other_secret_id)

    @decorators.idempotent_id('dff3d49e-9e31-46bb-b069-d4c72f591718')
    def test_get_other_project_secret_payload(self):
        other_secret_id = self.create_other_project_secret(
            'get_other_payload',
            payload='¡Más secreto!')
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_secret_payload,
            other_secret_id)

    @decorators.idempotent_id('fb2fe2a4-2ca9-4b64-b18f-cc0877eb27bc')
    def test_put_other_project_secret_payload(self):
        other_secret_id = self.create_other_project_secret('put_other_payload')
        self.assertRaises(
            exceptions.Forbidden,
            self.client.put_secret_payload,
            other_secret_id,
            'Shhhh... secret!')

    @decorators.idempotent_id('fc2f42ec-6bf4-4121-9698-4f0a7d01d8f3')
    def test_delete_other_project_secret(self):
        other_secret_id = self.create_other_project_secret(
            'get_other_payload',
            payload='loremipsumloremipsum')
        self.assertRaises(
            exceptions.Forbidden,
            self.client.delete_secret,
            other_secret_id)

    @decorators.idempotent_id('effafb29-fd10-41fb-9404-585af3de3602')
    def test_get_secret_acl(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_secret_acl,
            self.secret_id)

    @decorators.idempotent_id('d5058429-4e98-43ac-bda4-8160b2b95ef7')
    def test_put_secret_acl(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.put_secret_acl,
            self.secret_id,
            self.valid_acl)

    @decorators.idempotent_id('3350274a-b3f4-4178-927f-2591ef2dbea8')
    def test_patch_secret_acl(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.patch_secret_acl,
            self.secret_id,
            self.valid_acl)

    @decorators.idempotent_id('07104bf1-104b-4fa5-b855-82cba69bf24c')
    def test_delete_secret_acl(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.delete_secret_acl,
            self.secret_id)

    @decorators.idempotent_id('434187eb-1dd2-4544-bb1e-6be0dca8cd25')
    def test_get_other_secret_acl(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_secret_acl,
            self.other_secret_id)

    @decorators.idempotent_id('32312a70-8f02-4663-b7e0-4432950c2c11')
    def test_put_other_secret_acl(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.put_secret_acl,
            self.other_secret_id,
            self.valid_acl)

    @decorators.idempotent_id('ad95395f-45b0-4b34-b92a-e6c25f90e798')
    def test_patch_other_secret_acl(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.patch_secret_acl,
            self.other_secret_id,
            self.valid_acl)

    @decorators.idempotent_id('1470e2fc-46ce-4d06-a11a-201e9b5950c6')
    def test_delete_other_secret_acl(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.delete_secret_acl,
            self.other_secret_id)


class ProjectReaderV1_1Tests(ProjectReaderBase, BarbicanV1_1SecretConsumers):

    min_microversion = '1.1'

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.secret_consumer_client = \
            cls.os_project_reader.secret_v1_1.SecretConsumerClient()

    def setUp(self):
        super().setUp()
        self.test_consumer = {
            "service": "service1",
            "resource_id": "resource_id1",
            "resource_type": "resource_type1"
        }
        self.member_secret_consumer_client.add_consumer_to_secret(
            self.secret_id,
            **self.test_consumer
        )

    @decorators.idempotent_id('be85626b-ca83-4c90-9bf0-b918b9de21b6')
    def test_list_secret_consumers(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.secret_consumer_client.list_consumers_in_secret,
            self.secret_id)

    @decorators.idempotent_id('d7389369-62e9-4a25-b759-2a64f72fcba2')
    def test_create_secret_consumer(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.secret_consumer_client.add_consumer_to_secret,
            self.secret_id,
            **self.test_consumer)

    @decorators.idempotent_id('dbfba5e4-cd52-4ce9-bf50-a7e933ce5dcc')
    def test_delete_secret_consumer(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.secret_consumer_client.delete_consumer_from_secret,
            self.secret_id,
            **self.test_consumer)


class ProjectMemberTests(ProjectReaderTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.secret_client

    @decorators.idempotent_id('69f24625-0d8a-4412-b9c2-5a96fc689c87')
    def test_create_secret(self):
        """Test add_secret policy."""
        self.client.create_secret(name='test_create_secret')

        key = rbac_base.create_aes_key()
        expire_time = (datetime.utcnow() + timedelta(days=5))
        self.client.create_secret(
            name='test_create_secret2',
            expiration=expire_time.isoformat(), algorithm="aes",
            bit_length=256, mode="cbc", payload=key,
            payload_content_type="application/octet-stream",
            payload_content_encoding="base64"
        )

    @decorators.idempotent_id('ff89eaa4-0014-4935-80cf-f8b7970387e1')
    def test_list_secrets(self):
        """Test get_secrets policy."""
        # create two secrets
        self.create_empty_secret_admin('test_list_secrets')
        self.create_empty_secret_admin('test_list_secrets_2')

        # list secrets with name secret_1
        resp = self.client.list_secrets(name='test_list_secrets')
        secrets = resp['secrets']
        self.assertEqual('test_list_secrets', secrets[0]['name'])

        # list secrets with name secret_2
        resp = self.client.list_secrets(name='test_list_secrets_2')
        secrets = resp['secrets']
        self.assertEqual('test_list_secrets_2', secrets[0]['name'])

        # list all secrets
        resp = self.client.list_secrets()
        secrets = resp['secrets']
        self.assertGreaterEqual(len(secrets), 2)

    @decorators.idempotent_id('bffcf1e6-b9a2-43d8-b95c-b3683d6c4549')
    def test_delete_secret(self):
        """Test delete_secrets policy."""
        sec = self.create_empty_secret_admin('test_delete_secret_1')
        uuid = self.client.ref_to_uuid(sec['secret_ref'])
        self.client.delete_secret(uuid)

    @decorators.idempotent_id('72a59d44-1967-44fc-84a1-157a7bf124fa')
    def test_get_secret(self):
        """Test get_secret policy."""
        sec = self.create_empty_secret_admin('test_get_secret')
        uuid = self.client.ref_to_uuid(sec['secret_ref'])
        resp = self.client.get_secret_metadata(uuid)
        self.assertEqual(uuid, self.client.ref_to_uuid(resp['secret_ref']))

    @decorators.idempotent_id('cb848266-0172-4f93-add3-9d6f41a3bc46')
    def test_get_secret_payload(self):
        """Test get_secret payload policy."""
        key, sec = self.create_aes_secret_admin('test_get_secret_payload')
        uuid = self.client.ref_to_uuid(sec['secret_ref'])

        # Retrieve the payload
        payload = self.client.get_secret_payload(uuid)
        self.assertEqual(key, base64.b64encode(payload))

    @decorators.idempotent_id('f6c58ca1-50f2-4454-b3b2-b338a7dcf3cb')
    def test_put_secret_payload(self):
        """Test put_secret policy."""
        sec = self.create_empty_secret_admin('test_put_secret_payload')
        uuid = self.client.ref_to_uuid(sec['secret_ref'])

        key = rbac_base.create_aes_key()

        # Associate the payload with the created secret
        self.client.put_secret_payload(uuid, key)

        # Retrieve the payload
        payload = self.client.get_secret_payload(uuid)
        self.assertEqual(key, base64.b64encode(payload))

    @decorators.idempotent_id('63482006-18b5-40d3-82da-fdc078d6e5fe')
    def test_get_secret_acl(self):
        acl = self.client.get_secret_acl(self.secret_id)
        self.assertIn("read", acl.keys())

    @decorators.idempotent_id('915bdc2a-94d2-4835-a46e-a27f16ae57a2')
    def test_put_secret_acl(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.other_secret_client.get_secret_metadata,
            self.secret_id
        )
        _ = self.client.put_secret_acl(self.secret_id, self.valid_acl)
        acl = self.client.get_secret_acl(self.secret_id)
        self.assertIn(self.other_secret_client.user_id, acl['read']['users'])
        resp = self.other_secret_client.get_secret_metadata(self.secret_id)
        self.assertIn(self.secret_id, resp['secret_ref'])

    @decorators.idempotent_id('d6128f75-e7af-43dc-bf43-cf5fdc5f83be')
    def test_patch_secret_acl(self):
        _ = self.client.put_secret_acl(self.secret_id, self.valid_acl)
        acl = self.client.get_secret_acl(self.secret_id)
        self.assertIn(self.other_secret_client.user_id, acl['read']['users'])
        clear_users_acl = {
            'read': {
                'users': []
            }
        }
        _ = self.client.patch_secret_acl(self.secret_id, clear_users_acl)
        acl = self.client.get_secret_acl(self.secret_id)
        self.assertNotIn(self.other_secret_client.user_id,
                         acl['read']['users'])

    @decorators.idempotent_id('d566f1ab-c318-42cc-80d9-1ff9178d2c63')
    def test_delete_secret_acl(self):
        _ = self.client.put_secret_acl(self.secret_id, self.valid_acl)
        acl = self.client.get_secret_acl(self.secret_id)
        self.assertIn(self.other_secret_client.user_id, acl['read']['users'])

        _ = self.client.delete_secret_acl(self.secret_id)

        acl = self.client.get_secret_acl(self.secret_id)
        self.assertNotIn('users', acl['read'].keys())


class ProjectMemberV1_1Tests(ProjectReaderV1_1Tests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.secret_consumer_client = cls.member_secret_consumer_client

    @decorators.idempotent_id('ca2bbfa3-90b2-4f4e-8e57-3a3562d202a6')
    def test_list_secret_consumers(self):
        resp = self.secret_consumer_client.list_consumers_in_secret(
            self.secret_id
        )
        self.assertEqual(1, resp['total'])

    @decorators.idempotent_id('86cadb1e-f748-4d99-9477-4d171d4e9240')
    def test_create_secret_consumer(self):
        second_consumer = {
            'service': 'service2',
            'resource_id': 'resource_id2',
            'resource_type': 'resource_type2'
        }

        resp = self.secret_consumer_client.add_consumer_to_secret(
            self.secret_id,
            **second_consumer)

        self.assertEqual(2, len(resp['consumers']))

    @decorators.idempotent_id('f56c4c14-e8c7-4335-8d84-00f34355b53c')
    def test_delete_secret_consumer(self):
        resp = self.secret_consumer_client.delete_consumer_from_secret(
            self.secret_id,
            **self.test_consumer)

        self.assertEqual(0, len(resp['consumers']))


class ProjectAdminTests(ProjectMemberTests):
    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.admin_secret_client

    def test_delete_other_project_secret(self):
        other_secret_id = self.create_other_project_secret(
            'get_other_payload',
            payload='loremipsumloremipsum')
        self.client.delete_secret(other_secret_id)

    def test_get_other_project_secret(self):
        other_secret_id = self.create_other_project_secret(
            'get_other_secret',
            payload='¡Muy secreto!')
        self.client.get_secret_metadata(other_secret_id)


class ProjectAdminV1_1Tests(ProjectMemberV1_1Tests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.secret_consumer_client = cls.admin_secret_consumer_client

    @decorators.idempotent_id('2da9bfb4-f53b-45c0-b8c9-f657ced99bd4')
    def test_create_secret_consumer(self):
        pass

    @decorators.idempotent_id('2a4eaac5-76a1-48e2-b648-b1b02344130b')
    def test_delete_secret_consumer(self):
        pass

    @decorators.idempotent_id('34e2ded6-30ea-4f8a-b4e1-3aecac8fdd49')
    def test_list_secret_consumers(self):
        pass


class SystemReaderTests(rbac_base.BarbicanV1RbacBase, BarbicanV1RbacSecrets):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.secret_client

    @decorators.idempotent_id('104f71f0-8099-43ae-b4d9-cce5781a79b9')
    def test_create_secret(self):
        pass

    @decorators.idempotent_id('5a29b825-4f28-4733-90fa-579b63ae2b96')
    def test_list_secrets(self):
        pass

    @decorators.idempotent_id('b637c7db-64a9-46c8-b322-c4d282e05164')
    def test_delete_secret(self):
        pass

    @decorators.idempotent_id('8d0a7f54-61f3-432e-8f4b-c04945b40373')
    def test_get_secret(self):
        pass

    @decorators.idempotent_id('1bc76c3a-a69e-4285-8f8e-f7bacd01fef8')
    def test_get_secret_payload(self):
        pass

    @decorators.idempotent_id('c2f38e3d-cc52-43c0-9fb3-8065797c40da')
    def test_put_secret_payload(self):
        pass

    @decorators.idempotent_id('be392729-af43-4aab-bbc7-43fcd5df9140')
    def test_get_other_project_secret(self):
        pass

    @decorators.idempotent_id('cc021881-7fba-48a2-aa6e-68c426b382f9')
    def test_get_other_project_secret_payload(self):
        pass

    @decorators.idempotent_id('e87f5e40-7bb3-4fc8-aa5a-23cc1a8850f5')
    def test_put_other_project_secret_payload(self):
        pass

    @decorators.idempotent_id('ce878824-d424-4abb-8217-068c9a99333b')
    def test_delete_other_project_secret(self):
        pass

    @decorators.idempotent_id('be04944b-b4e2-4f66-b58a-3d047c99d939')
    def test_get_secret_acl(self):
        pass

    @decorators.idempotent_id('65ce0063-d6f1-463c-b752-d4871a9df684')
    def test_put_secret_acl(self):
        pass

    @decorators.idempotent_id('81423acc-240c-46f0-8de9-4cf6ab5d4bc4')
    def test_patch_secret_acl(self):
        pass

    @decorators.idempotent_id('d55d5798-c23f-4108-8005-963d350d9d41')
    def test_delete_secret_acl(self):
        pass

    @decorators.idempotent_id('5490d517-ba6c-4e28-8712-07dbc9bb9ada')
    def test_get_other_secret_acl(self):
        pass

    @decorators.idempotent_id('1ae61619-104a-4497-999c-00671335bc4f')
    def test_put_other_secret_acl(self):
        pass

    @decorators.idempotent_id('5ec28567-111b-405f-93c9-ed5d4259e918')
    def test_patch_other_secret_acl(self):
        pass

    @decorators.idempotent_id('65cfefb4-69a6-4f31-b8e1-defad1b57645')
    def test_delete_other_secret_acl(self):
        pass


class SystemMemberTests(SystemReaderTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.secret_client


class SystemAdminTests(SystemMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.secret_client



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import abc

from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

from barbican_tempest_plugin.tests.rbac.v1 import base


class BarbicanV1RbacSecretMetadata:

    @abc.abstractmethod
    def test_create_key_value_pair(self):
        """Test create_key_value_pair policy

        Testing: POST /v1/secrets/{secret-id}/metadata
        This test must check:
          * whether the persona can add metadata to a secret
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_put_secret_metadata(self):
        """Test put_secret_metadata policy

        Testing: PUT /v1/secrets/{secret-id}/metadata
        This test must check:
          * whether the persona can update metadata on a secret
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_secret_metadata(self):
        """Test get_secret_metadata policy

        Testing: GET /v1/secrets/{secret-id}/metadata
        This test must check:
          * whether the persona can retrieve secret metadata
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_update_secret_metadata_by_key(self):
        """Test update_secret_metadata policy

        Testing: PUT /v1/secrets/{secret-id}/metadata/{meta-key}
        This test must check:
          * whether the persona can update individual secret metadata
            values
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_secret_metadata_by_key(self):
        """Test get_secret_metadata_by_key policy

        Testing: GET /v1/secrets/{secret-id}/metadata/{meta-key}
        This test must check:
          * whether the persona can retrieve individual secret metadata
            values
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_secret_metadata_by_key(self):
        """Test delete_secret_metadata_by_key policy

        Testing: DELETE /v1/secrets/{secret-id}/metadata/{meta-key}
        This test must check:
          * whether the persona can delete individual secret metadata
            values
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_create_key_value_pair_on_other_secret(self):
        """Test create_key_value_pair policy

        Testing: POST /v1/secrets/{secret-id}/metadata
        This test must check:
          * whether the persona can add metadata to a secret
            that belongs to a different project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_put_secret_metadata_on_other_secret(self):
        """Test put_secret_metadata policy

        Testing: PUT /v1/secrets/{secret-id}/metadata
        This test must check:
          * whether the persona can update metadata on a secret
            that belongs to a different project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_secret_metadata_from_other_secret(self):
        """Test get_secret_metadata policy

        Testing: GET /v1/secrets/{secret-id}/metadata
        This test must check:
          * whether the persona can retrieve secret metadata
            from a secret that belongs to a different project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_update_other_secret_metadata_by_key(self):
        """Test update_secret_metadata policy

        Testing: PUT /v1/secrets/{secret-id}/metadata/{meta-key}
        This test must check:
          * whether the persona can update individual secret metadata
            values on a secret that belongs to a different project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_other_secret_metadata_by_key(self):
        """Test get_secret_metadata_by_key policy

        Testing: GET /v1/secrets/{secret-id}/metadata/{meta-key}
        This test must check:
          * whether the persona can retrieve individual secret metadata
            values for a secret that belongs to a different project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_other_secret_metadata_by_key(self):
        """Test delete_secret_metadata policy

        Testing: DELETE /v1/secrets/{secret-id}/metadata/{meta-key}
        This test must check:
          * whether the persona can delete individual secret metadata
            values
        """
        raise NotImplementedError


class ProjectReaderTests(base.BarbicanV1RbacBase,
                         BarbicanV1RbacSecretMetadata):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.secret_v1.SecretMetadataClient()

    def setUp(self):
        super().setUp()
        self.secret_id = self.create_test_secret(
            self.secret_client,
            data_utils.rand_name('test-secret-metadata'),
            'SECRET_PASSPHRASE')
        self.secret_metadata_client.create_key_value_pair(
            self.secret_id,
            'foo',
            'bar')

        self.other_secret_id = self.create_test_secret(
            self.other_secret_client,
            data_utils.rand_name('test-secret-metadata'),
            'SECRET_PASSPHRASE')
        self.other_secret_metadata_client.create_key_value_pair(
            self.other_secret_id,
            'foo',
            'bar')

    @decorators.idempotent_id('2dd1cb04-67e9-4c3c-a40e-12184eff5bc6')
    def test_create_key_value_pair(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.create_key_value_pair,
            self.secret_id,
            'mykey',
            'foo'
        )

    @decorators.idempotent_id('33c1d271-5367-4f0e-a2e7-c28ea1130fa6')
    def test_put_secret_metadata(self):
        meta = {
            'foo': 'bar',
            'baz': 'bork'
        }
        self.assertRaises(
            exceptions.Forbidden,
            self.client.put_secret_metadata,
            self.secret_id,
            **meta)

    @decorators.idempotent_id('8f22488b-52d5-4a28-ae32-faf1514ef390')
    def test_get_secret_metadata(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_secret_metadata,
            self.secret_id)

    @decorators.idempotent_id('df6bbafc-f836-44b4-a0a7-3d0f94f5b9ac')
    def test_update_secret_metadata_by_key(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.update_secret_metadata,
            self.secret_id,
            'foo',
            'baz')

    @decorators.idempotent_id('b2f3bafb-4a21-47da-966f-3e3010571596')
    def test_get_secret_metadata_by_key(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_secret_metadata_by_key,
            self.secret_id,
            'foo')

    @decorators.idempotent_id('0d80db72-fc6f-445f-b402-f86c91233b4f')
    def test_delete_secret_metadata_by_key(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.delete_secret_metadata_by_key,
            self.secret_id,
            'foo')

    @decorators.idempotent_id('7c6223b6-5a2f-4989-ae54-db253702af98')
    def test_create_key_value_pair_on_other_secret(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.create_key_value_pair,
            self.other_secret_id,
            'mykey',
            'foo'
        )

    @decorators.idempotent_id('c1738dd5-e67a-4b48-8064-c198ca0a7970')
    def test_put_secret_metadata_on_other_secret(self):
        meta = {
            'foo': 'bar',
            'baz': 'bork'
        }
        self.assertRaises(
            exceptions.Forbidden,
            self.client.put_secret_metadata,
            self.other_secret_id,
            **meta)

    @decorators.idempotent_id('efb8d207-5418-4c3a-bb30-d1d5e1695c41')
    def test_get_secret_metadata_from_other_secret(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_secret_metadata,
            self.other_secret_id)

    @decorators.idempotent_id('489d52ae-3324-420b-b69c-938f2eb41f6f')
    def test_update_other_secret_metadata_by_key(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.update_secret_metadata,
            self.other_secret_id,
            'foo',
            'baz')

    @decorators.idempotent_id('f2cbaec3-94ff-4a9c-bc2d-4c728f46313b')
    def test_get_other_secret_metadata_by_key(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_secret_metadata_by_key,
            self.other_secret_id,
            'foo')

    @decorators.idempotent_id('fb581f6c-1d59-420c-b05b-227514c75789')
    def test_delete_other_secret_metadata_by_key(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.delete_secret_metadata_by_key,
            self.other_secret_id,
            'foo')


class ProjectMemberTests(ProjectReaderTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.secret_metadata_client

    @decorators.idempotent_id('73d4c9cc-b7ad-4706-bec4-e7dc70698711')
    def test_create_key_value_pair(self):
        resp = self.client.create_key_value_pair(
            self.secret_id,
            'mykey',
            'foo'
        )
        self.assertEqual('mykey', resp['key'])
        self.assertEqual('foo', resp['value'])

    @decorators.idempotent_id('f7178757-5ae4-4041-a57c-322f7665f055')
    def test_put_secret_metadata(self):
        test_meta = {
            'foo': 'baz',
            'bar': 'bork'
        }
        self.client.put_secret_metadata(self.secret_id, **test_meta)
        resp = self.client.get_secret_metadata(self.secret_id)

        self.assertIn('bar', resp.keys())
        self.assertEqual('baz', resp['foo'])

    @decorators.idempotent_id('054348d4-c4b4-446a-ad98-c79f6de42eec')
    def test_get_secret_metadata(self):
        resp = self.client.get_secret_metadata(self.secret_id)

        self.assertIn('foo', resp.keys())
        self.assertEqual('bar', resp['foo'])

    @decorators.idempotent_id('7ed07736-39e7-4c4f-b5c9-f59017f3e80b')
    def test_update_secret_metadata_by_key(self):
        self.client.update_secret_metadata(self.secret_id, 'foo', 'baz')

        resp = self.secret_metadata_client.get_secret_metadata(self.secret_id)
        self.assertEqual('baz', resp['foo'])

    @decorators.idempotent_id('26160af4-ff17-4023-9238-a2a9dca9946c')
    def test_get_secret_metadata_by_key(self):
        resp = self.client.get_secret_metadata_by_key(self.secret_id, 'foo')
        self.assertEqual('foo', resp['key'])
        self.assertEqual('bar', resp['value'])

    @decorators.idempotent_id('3740d6ec-304f-43ce-aa54-af62006715d8')
    def test_delete_secret_metadata_by_key(self):
        self.client.delete_secret_metadata_by_key(self.secret_id, 'foo')
        self.assertRaises(
            exceptions.NotFound,
            self.client.get_secret_metadata_by_key,
            self.secret_id,
            'foo')


class ProjectAdminTests(ProjectMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.admin_secret_metadata_client



# Copyright (c) 2017 Johns Hopkins University Applied Physics Laboratory
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import json

from tempest import config

from barbican_tempest_plugin.services.key_manager.json import base

CONF = config.CONF


class SecretMetadataClient(base.BarbicanTempestClient):

    def get_secret_metadata(self, secret_id):
        resp, body = self.get("v1/secrets/%s/metadata" % secret_id)
        self.expected_success(200, resp.status)
        # Note: "metadata" top level key gets dropped by _parse_resp()
        return self._parse_resp(body)

    def put_secret_metadata(self, secret_id, **kwargs):
        body_dict = {'metadata': kwargs}
        uri = "v1/secrets/%s/metadata" % secret_id
        resp, body = self.put(uri, json.dumps(body_dict))
        self.expected_success(201, resp.status)
        # Note: "metadata" top level key gets dropped by _parse_resp()
        return self._parse_resp(body)

    def get_secret_metadata_by_key(self, secret_id, key):
        uri = "v1/secrets/{uuid}/metadata/{key}".format(uuid=secret_id,
                                                        key=key)
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        return self._parse_resp(body)

    def create_key_value_pair(self, secret_id, key, value):
        body_dict = {
            'key': key,
            'value': value
        }
        resp, body = self.post("v1/secrets/%s/metadata" % secret_id,
                               json.dumps(body_dict))
        self.expected_success(201, resp.status)
        return self._parse_resp(body)

    def update_secret_metadata(self, secret_id, key, value):
        uri = "v1/secrets/{uuid}/metadata/{key}".format(uuid=secret_id,
                                                        key=key)
        body_dict = {
            'key': key,
            'value': value
        }
        resp, body = self.put(uri, json.dumps(body_dict))
        self.expected_success(200, resp.status)
        return self._parse_resp(body)

    def delete_secret_metadata_by_key(self, secret_id, key):
        uri = "v1/secrets/{uuid}/metadata/{key}".format(uuid=secret_id,
                                                        key=key)
        resp, body = self.delete(uri)
        self.expected_success(204, resp.status)
        return self._parse_resp(body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from barbican_tempest_plugin.tests.rbac.v1 import base
from oslo_log import log as logging
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions


CONF = config.CONF
LOG = logging.getLogger(__name__)


class CVE20223100Test(base.BarbicanV1RbacBase):

    @decorators.idempotent_id('459159ef-9670-4c59-8528-09466185c84e')
    def test_cve_2022_3100(self):
        # create a secret that belongs to Project B
        secret_id = self.create_test_secret(
            self.other_secret_client,
            data_utils.rand_name('secret-under-test'),
            'DONT_CVE_ME_PLZ')

        # attempt to retrieve secret payload with user from Project A
        # using CVE exploit (e.g. by adding the query string
        # ?target.secret.read=read to the request)
        query = {'target.secret.read': 'read'}
        self.assertRaises(
            exceptions.Forbidden,
            self.secret_client.get_secret_payload,
            secret_id,
            **query)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import base64
from datetime import datetime
from datetime import timedelta
import os

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC

from tempest import clients
from tempest import config
from tempest.lib import auth
from tempest.lib.common import api_version_utils
from tempest.lib.common.utils import data_utils
from tempest import test


CONF = config.CONF

RESOURCE_TYPES = ['container', 'order', 'quota', 'secret']


def create_aes_key():
    password = b"password"
    salt = os.urandom(16)
    kdf = PBKDF2HMAC(
        algorithm=hashes.SHA256(), length=32, salt=salt,
        iterations=1000, backend=default_backend()
    )
    return base64.b64encode(kdf.derive(password))


class BarbicanV1RbacBase(test.BaseTestCase,
                         api_version_utils.BaseMicroversionTest):

    identity_version = 'v3'
    _created_projects = None
    _created_users = None
    created_objects = {}

    credentials = [
        'system_admin',
        'project_alt_member'
    ]

    # TODO(dmendiza): remove this and use the clients instead
    @classmethod
    def ref_to_uuid(cls, href):
        return href.split('/')[-1]

    @classmethod
    def skip_checks(cls):
        super().skip_checks()
        if not CONF.enforce_scope.barbican:
            raise cls.skipException("enforce_scope is not enabled for "
                                    "barbican, skipping RBAC tests")
        api_version_utils.check_skip_with_microversion(
            cls.min_microversion,
            cls.max_microversion,
            CONF.key_manager.min_microversion,
            CONF.key_manager.max_microversion)

    @classmethod
    def setup_credentials(cls):
        super().setup_credentials()
        cls._created_projects = list()
        cls._created_users = list()
        project_id = cls.os_system_admin.projects_client.create_project(
            data_utils.rand_name()
        )['project']['id']
        cls._created_projects.append(project_id)
        cls.os_project_admin = cls._setup_new_user_client(project_id, 'admin')
        cls.os_project_member = cls._setup_new_user_client(project_id,
                                                           'member')
        cls.os_project_other_member = cls._setup_new_user_client(project_id,
                                                                 'member')
        cls.os_project_reader = cls._setup_new_user_client(project_id,
                                                           'reader')

    @classmethod
    def _setup_new_user_client(cls, project_id, role):
        """Create a new tempest.clients.Manager

        Creates a new user with the given roles on the given project,
        and returns an instance of tempest.clients.Manager set up
        for that user.

        Users are cleaned up during class teardown in cls.clear_credentials
        """
        user = {
            'name': data_utils.rand_name('user'),
            'password': data_utils.rand_password()
        }
        user_id = cls.os_system_admin.users_v3_client.create_user(
            **user
        )['user']['id']
        cls._created_users.append(user_id)
        role_id = cls.os_system_admin.roles_v3_client.list_roles(
            name=role
        )['roles'][0]['id']
        cls.os_system_admin.roles_v3_client.create_user_role_on_project(
            project_id, user_id, role_id
        )
        creds = auth.KeystoneV3Credentials(
            user_id=user_id,
            password=user['password'],
            project_id=project_id
        )
        auth_provider = clients.get_auth_provider(creds)
        creds = auth_provider.fill_credentials()
        return clients.Manager(credentials=creds)

    @classmethod
    def clear_credentials(cls):
        for user_id in cls._created_users:
            cls.os_system_admin.users_v3_client.delete_user(user_id)
        for project_id in cls._created_projects:
            cls.os_system_admin.projects_client.delete_project(project_id)
        super().clear_credentials()

    @classmethod
    def setup_clients(cls):
        super().setup_clients()

        # setup clients for admin persona
        adm = cls.os_project_admin
        cls.admin_secret_client = adm.secret_v1.SecretClient()
        cls.admin_secret_metadata_client = adm.secret_v1.SecretMetadataClient()
        cls.admin_consumer_client = adm.secret_v1.ConsumerClient()
        cls.admin_secret_consumer_client = \
            adm.secret_v1_1.SecretConsumerClient()
        cls.admin_container_client = adm.secret_v1.ContainerClient()
        cls.admin_order_client = adm.secret_v1.OrderClient(
            secret_client=cls.admin_secret_client,
            container_client=cls.admin_container_client
        )
        cls.admin_quota_client = adm.secret_v1.QuotaClient()

        # set clients for member persona
        member = cls.os_project_member
        cls.secret_client = member.secret_v1.SecretClient()
        cls.secret_metadata_client = member.secret_v1.SecretMetadataClient()
        cls.member_consumer_client = member.secret_v1.ConsumerClient()
        cls.member_secret_consumer_client = \
            member.secret_v1_1.SecretConsumerClient()
        cls.container_client = member.secret_v1.ContainerClient()
        cls.order_client = member.secret_v1.OrderClient(
            secret_client=cls.secret_client,
            container_client=cls.container_client
        )
        cls.quota_client = member.secret_v1.QuotaClient()

        # set up clients for member persona associated with a different
        # project
        cls.other_secret_client = \
            cls.os_project_alt_member.secret_v1.SecretClient()
        cls.other_secret_metadata_client = \
            cls.os_project_alt_member.secret_v1.SecretMetadataClient()
        cls.other_container_client = \
            cls.os_project_alt_member.secret_v1.ContainerClient()
        cls.other_order_client = \
            cls.os_project_alt_member.secret_v1.OrderClient(
                secret_client=cls.other_secret_client,
                container_client=cls.other_container_client
            )

    @classmethod
    def resource_setup(cls):
        super().resource_setup()
        for resource in RESOURCE_TYPES:
            cls.created_objects[resource] = set()

    @classmethod
    def resource_cleanup(cls):
        try:
            for container_uuid in list(cls.created_objects['container']):
                cls.admin_container_client.delete_container(container_uuid)
                cls.created_objects['container'].remove(container_uuid)
            for quota_uuid in list(cls.created_objects['quota']):
                cls.admin_quota_client.delete_project_quota(quota_uuid)
                cls.created_objects['quota'].remove(quota_uuid)
            for secret_uuid in list(cls.created_objects['secret']):
                cls.admin_secret_client.delete_secret(secret_uuid)
                cls.created_objects['secret'].remove(secret_uuid)

            for client in [cls.secret_client,
                           cls.order_client,
                           cls.admin_secret_client,
                           cls.admin_order_client,
                           cls.other_secret_client,
                           cls.other_order_client]:
                client.cleanup()
        finally:
            super(BarbicanV1RbacBase, cls).resource_cleanup()

    @classmethod
    def add_cleanup(cls, resource, response):
        if resource == 'container':
            uuid = cls.ref_to_uuid(response['container_ref'])
        if resource == 'quota':
            uuid = cls.ref_to_uuid(response['quota_ref'])
        if resource == 'secret':
            uuid = cls.ref_to_uuid(response['secret_ref'])
        cls.created_objects[resource].add(uuid)

    @classmethod
    def delete_cleanup(cls, resource, uuid):
        cls.created_objects[resource].remove(uuid)

    # TODO(dmendiza): get rid of this helper method.
    def do_request(self, method, client=None, expected_status=200,
                   cleanup=None, **args):
        if client is None:
            client = self.client
        if isinstance(expected_status, type(Exception)):
            self.assertRaises(expected_status,
                              getattr(client, method),
                              **args)
        else:
            response = getattr(client, method)(**args)
            # self.assertEqual(response.response.status, expected_status)
            if cleanup is not None:
                self.add_cleanup(cleanup, response)
            return response

    def create_empty_secret_admin(self, secret_name):
        """add empty secret as admin user """
        return self.admin_secret_client.create_secret(name=secret_name)

    def create_empty_container_admin(self,
                                     container_name,
                                     container_type='generic'):
        """add empty container as admin user"""
        return self.admin_container_client.create_container(
            name=container_name,
            type=container_type)

    def create_aes_secret_admin(self, secret_name):
        key = create_aes_key()
        expire_time = (datetime.utcnow() + timedelta(days=5))
        return key, self.do_request(
            'create_secret', client=self.admin_secret_client,
            expected_status=201, cleanup="secret",
            expiration=expire_time.isoformat(), algorithm="aes",
            bit_length=256, mode="cbc", payload=key,
            payload_content_type="application/octet-stream",
            payload_content_encoding="base64",
            name=secret_name
        )

    def create_other_project_secret(self, secret_name, payload=None):
        kwargs = {
            'name': secret_name,
            'secret_type': 'passphrase',
        }
        if payload is not None:
            kwargs['payload'] = payload
            kwargs['payload_content_type'] = 'text/plain'
        resp = self.other_secret_client.create_secret(**kwargs)
        return self.other_secret_client.ref_to_uuid(resp['secret_ref'])

    def create_test_secret(self, client, name, payload=None):
        """Create a secret for testing

        The new secret is created using the given client.  If no
        payload is given, the secret is left empty.

        :returns: the uuid for the new secret
        """
        kwargs = {
            'name': name,
            'secret_type': 'passphrase'
        }
        if payload is not None:
            kwargs['payload'] = payload
            kwargs['payload_content_type'] = 'text/plain'
        resp = client.create_secret(**kwargs)
        return client.ref_to_uuid(resp['secret_ref'])

    def create_test_order(self, client, order_name):
        """Create a symmetric key order for testing

        The new order is created using the given
        client.

        :returns: the uuid for the new order
        """
        kwargs = {
            'type': 'key',
            'meta': {
                'name': order_name,
                'algorithm': 'AES',
                'bit_length': 256,
                'mode': 'CBC',
            }
        }
        resp = client.create_order(**kwargs)
        return client.ref_to_uuid(resp['order_ref'])

    def create_test_container(self, client, name):
        """Create a generic container for testing

        The new container is created using the given client.

        :returns: the uuid for the new container
        """
        container = {
            "type": "generic",
            "name": name,
        }
        resp = client.create_container(**container)
        return client.ref_to_uuid(resp['container_ref'])



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import abc

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions

from barbican_tempest_plugin.tests.rbac.v1 import base


CONF = config.CONF


class BarbicanV1RbacSecretStores:

    @abc.abstractmethod
    def test_list_secret_stores(self):
        """Test getting a list of all backends

        Testing: GET /v1/secret-stores
        This test must check:
          * whether the persona can list all secret stores
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_secret_store(self):
        """Test get secret store information

        Testing: GET /v1/secret-stores/{secret-store-id}
        This test must check:
          * whether the persona can get information about a specific
            secret store
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_global_secret_store(self):
        """Test getting the global secret store

        Testing: GET /v1/secret-stores/global-default
        This test must check:
          * whether the persona can get information about the global
            default secret store
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_preferred_secret_store(self):
        """Test getting the preferred secret store

        Testing: GET /v1/secret-stores/preferred
        This test must check:
          * whether the persona can get information about their project's
            preferred secret store
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_set_preferred_secret_store(self):
        """Test setting the preferred secret store

        Testing: POST /v1/secret-stores/{secret-store-id}/preferred
        This test must check:
          * whether the persona can set their project's preferred
            secret store
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_unset_preferred_secret_store(self):
        """Test removing the preferred secret store

        Testing: DELETE /v1/secret-stores/{secret-store-id}/preferred
        This test must check:
          * whether the persona can set their project's preferred
            secret store
        """
        raise NotImplementedError


class ProjectMemberTests(base.BarbicanV1RbacBase, BarbicanV1RbacSecretStores):

    @classmethod
    def skip_checks(cls):
        super().skip_checks()
        if not CONF.barbican_tempest.enable_multiple_secret_stores:
            raise cls.skipException("enable_multiple_secret_stores is not "
                                    "configured.  Skipping RBAC tests.")

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_member.secret_v1.SecretStoresClient()

    @decorators.idempotent_id('bf31560c-42e5-4afc-b1d6-a3a8aa7773d3')
    def test_list_secret_stores(self):
        resp = self.do_request('list_secret_stores')
        self.assertIn('secret_stores', resp)

    @decorators.idempotent_id('28d8e43b-af38-4985-8f70-3c1098116561')
    def test_get_secret_store(self):
        resp = self.do_request('list_secret_stores')
        secret_store_id = self.ref_to_uuid(
            resp['secret_stores'][0]['secret_store_ref']
        )
        resp = self.do_request('get_secret_store',
                               secret_store_id=secret_store_id)
        self.assertEqual(secret_store_id,
                         self.ref_to_uuid(resp['secret_store_ref']))

    @decorators.idempotent_id('e97d0cbe-112c-490e-b3d7-02981161d471')
    def test_get_global_secret_store(self):
        resp = self.do_request('get_global_secret_store')
        self.assertTrue(resp['global_default'])

    @decorators.idempotent_id('c3554210-8960-4d09-a1ba-369b8df6ca1f')
    def test_get_preferred_secret_store(self):
        # First use project admin to set preferred secret store
        resp = self.do_request('list_secret_stores')
        secret_store_id = self.ref_to_uuid(
            resp['secret_stores'][0]['secret_store_ref']
        )
        admin_client = self.os_project_admin.secret_v1.SecretStoresClient()
        self.do_request('set_preferred_secret_store',
                        client=admin_client,
                        secret_store_id=secret_store_id)

        # Check that other users in project can view the newly set
        # preferred secret store
        resp = self.do_request('get_preferred_secret_store')
        self.assertEqual('ACTIVE', resp['status'])

    @decorators.idempotent_id('ada28e3a-ec67-4994-9dde-410463d6d06e')
    def test_set_preferred_secret_store(self):
        resp = self.do_request('list_secret_stores')
        secret_store_id = self.ref_to_uuid(
            resp['secret_stores'][0]['secret_store_ref']
        )
        self.do_request('set_preferred_secret_store',
                        expected_status=exceptions.Forbidden,
                        secret_store_id=secret_store_id)

    @decorators.idempotent_id('c3f52fd1-5d18-498f-81b2-45df5cb09a87')
    def test_unset_preferred_secret_store(self):
        resp = self.do_request('list_secret_stores')
        secret_store_id = self.ref_to_uuid(
            resp['secret_stores'][0]['secret_store_ref']
        )
        self.do_request('unset_preferred_secret_store',
                        expected_status=exceptions.Forbidden,
                        secret_store_id=secret_store_id)


class ProjectAdminTests(ProjectMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_admin.secret_v1.SecretStoresClient()

    @decorators.idempotent_id('c459deb6-6447-43c4-820b-384903e700cb')
    def test_set_preferred_secret_store(self):
        resp = self.do_request('list_secret_stores')
        secret_store_id = self.ref_to_uuid(
            resp['secret_stores'][0]['secret_store_ref']
        )
        self.do_request('set_preferred_secret_store',
                        secret_store_id=secret_store_id)
        resp = self.do_request('get_preferred_secret_store')
        self.assertEqual(secret_store_id,
                         self.ref_to_uuid(resp['secret_store_ref']))

    @decorators.idempotent_id('d21ca9b6-b62e-43d1-9c9f-a6e16c939c01')
    def test_unset_preferred_secret_store(self):
        resp = self.do_request('list_secret_stores')
        secret_store_id = self.ref_to_uuid(
            resp['secret_stores'][0]['secret_store_ref']
        )
        self.do_request('set_preferred_secret_store',
                        secret_store_id=secret_store_id)
        self.do_request('unset_preferred_secret_store',
                        secret_store_id=secret_store_id)
        self.do_request('get_preferred_secret_store',
                        expected_status=exceptions.NotFound)


class ProjectReaderTests(ProjectMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.secret_v1.SecretStoresClient()



# Copyright (c) 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from tempest.lib import decorators

from barbican_tempest_plugin.tests.api import base


class SecretConsumersTest(base.BaseKeyManagerTest):
    """Secret Consumers API tests."""

    min_microversion = '1.1'

    @decorators.idempotent_id('07a47f8b-e454-4dd0-afb6-bfa12677cd8e')
    def test_add_delete_consumers_in_secret(self):
        # Create a secret to test against
        sec = self.create_secret(name='secret_1')
        secret_id = self.secret_consumer_client.ref_to_uuid(sec['secret_ref'])

        # Confirm that the secret has no consumers
        body = self.secret_consumer_client.list_consumers_in_secret(secret_id)
        self.assertEqual(0, body.get('total'))
        self.assertEmpty(body.get('consumers'))

        # Add some consumers to the secret
        body = self.secret_consumer_client.add_consumer_to_secret(
            secret_id,
            service="service1",
            resource_id="resource_id1",
            resource_type="resource_type1"
        )
        self.assertEqual(
            secret_id,
            self.secret_consumer_client.ref_to_uuid(body.get('secret_ref'))
        )
        self.assertEqual(1, len(body.get('consumers')))
        body = self.secret_consumer_client.add_consumer_to_secret(
            secret_id,
            service="service2",
            resource_id="resource_id2",
            resource_type="resource_type2"
        )
        self.assertEqual(
            secret_id,
            self.secret_consumer_client.ref_to_uuid(body.get('secret_ref'))
        )
        self.assertEqual(2, len(body.get('consumers')))

        # Confirm that the consumers are in the secret
        body = self.secret_consumer_client.list_consumers_in_secret(secret_id)
        self.assertEqual(2, body.get('total'))
        self.assertEqual(2, len(body.get('consumers')))
        for consumer in body.get('consumers'):
            self.assertIn(consumer.get('service'), ("service1", "service2"))
            self.assertIn(consumer.get('resource_id'),
                          ("resource_id1", "resource_id2"))
            self.assertIn(consumer.get('resource_type'),
                          ("resource_type1", "resource_type2"))

        # Remove the consumers from the secret
        body = self.secret_consumer_client.delete_consumer_from_secret(
            secret_id,
            service="service1",
            resource_id="resource_id1",
            resource_type="resource_type1"
        )
        self.assertEqual(
            secret_id,
            self.secret_consumer_client.ref_to_uuid(body.get('secret_ref'))
        )
        self.assertEqual(1, len(body.get('consumers')))
        body = self.secret_consumer_client.delete_consumer_from_secret(
            secret_id,
            service="service2",
            resource_id="resource_id2",
            resource_type="resource_type2"
        )
        self.assertEqual(
            secret_id,
            self.secret_consumer_client.ref_to_uuid(body.get('secret_ref'))
        )
        self.assertEqual(0, len(body.get('consumers')))

        # Confirm that the secret has no consumers
        body = self.secret_consumer_client.list_consumers_in_secret(secret_id)
        self.assertEqual(0, body.get('total'))
        self.assertEqual(0, len(body.get('consumers')))

        # Clean up the secret
        self.delete_secret(secret_id)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json
from urllib import parse

from barbican_tempest_plugin.services.key_manager.json import base


class SecretStoresClient(base.BarbicanTempestClient):

    def list_secret_stores(self, **kwargs):
        uri = '/v1/secret-stores'
        if kwargs:
            uri += '?{}'.format(parse.urlencode(kwargs))
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        return json.loads(body.decode('UTF-8'))

    def get_secret_store(self, secret_store_id):
        uri = '/v1/secret-stores/{}'.format(secret_store_id)
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        return json.loads(body.decode('UTF-8'))

    def get_global_secret_store(self, **kwargs):
        uri = '/v1/secret-stores/global-default'
        if kwargs:
            uri += '?{}'.format(parse.urlencode(kwargs))
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        return json.loads(body.decode('UTF-8'))

    def get_preferred_secret_store(self, **kwargs):
        uri = '/v1/secret-stores/preferred'
        if kwargs:
            uri += '?{}'.format(parse.urlencode(kwargs))
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        return json.loads(body.decode('UTF-8'))

    def set_preferred_secret_store(self, secret_store_id):
        uri = '/v1/secret-stores/{}/preferred'.format(secret_store_id)
        resp, body = self.post(uri, None)
        self.expected_success(204, resp.status)

    def unset_preferred_secret_store(self, secret_store_id):
        uri = '/v1/secret-stores/{}/preferred'.format(secret_store_id)
        resp, body = self.delete(uri)
        self.expected_success(204, resp.status)



# Copyright (c) 2022 Red Hat Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import json

from urllib import parse as urllib

from tempest import config

from barbican_tempest_plugin.services.key_manager.json import base


CONF = config.CONF


class SecretConsumerClient(base.BarbicanTempestClient):

    _microversion = '1.1'

    def list_consumers_in_secret(self, secret_id, **kwargs):
        uri = "/v1/secrets/%s/consumers" % secret_id
        if kwargs:
            uri += "?%s" % urllib.urlencode(kwargs)

        response, body = self.get(uri)
        self.expected_success(200, response.status)
        return json.loads(body.decode("utf-8"))

    def add_consumer_to_secret(self, secret_id, **kwargs):
        uri = "/v1/secrets/%s/consumers" % secret_id

        response, body = self.post(uri, json.dumps(kwargs))
        self.expected_success(200, response.status)
        return json.loads(body.decode("utf-8"))

    def delete_consumer_from_secret(self, secret_id, **kwargs):
        uri = "/v1/secrets/%s/consumers" % secret_id

        response, body = self.delete(uri, body=json.dumps(kwargs))
        self.expected_success(200, response.status)
        return json.loads(body.decode("utf-8"))



# Copyright 2017 Johns Hopkins Applied Physics Lab
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators

from barbican_tempest_plugin.tests.api import base


class SecretMetadataTest(base.BaseKeyManagerTest):
    """Metadata API test"""
    @decorators.idempotent_id('2b0c1707-afc3-4674-a6c6-4dc42f318117')
    def test_secret_metadata(self):
        # Create a secret
        sec = self.create_secret()
        uuid = base._get_uuid(sec['secret_ref'])

        # Add multiple metadata fields
        self.secret_metadata_client.put_secret_metadata(
            uuid,
            description='contains the AES key',
            geolocation='12.3456, -98.7654'
        )

        metadata = self.secret_metadata_client.get_secret_metadata(uuid)
        self.assertEqual(2, len(metadata.keys()))
        self.assertIn('description', metadata.keys())
        self.assertIn('geolocation', metadata.keys())
        self.assertEqual('contains the AES key', metadata['description'])
        self.assertEqual('12.3456, -98.7654', metadata['geolocation'])

        # Add a single metadata field
        self.secret_metadata_client.create_key_value_pair(
            uuid,
            key='extra',
            value='extra value'
        )
        metadata = self.secret_metadata_client.get_secret_metadata(uuid)
        self.assertEqual(3, len(metadata.keys()))
        self.assertEqual('extra value', metadata['extra'])

        # Modify the metadata field
        self.secret_metadata_client.update_secret_metadata(
            uuid,
            key='extra',
            value='new value'
        )
        metadata = self.secret_metadata_client.get_secret_metadata(uuid)
        self.assertEqual('new value', metadata['extra'])

        # Delete the extra key-value pair
        self.secret_metadata_client.delete_secret_metadata_by_key(
            uuid,
            'extra'
        )
        metadata = self.secret_metadata_client.get_secret_metadata(uuid)
        self.assertEqual(2, len(metadata.keys()))



# Copyright (c) 2016 Johns Hopkins University Applied Physics Laboratory
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json
from urllib import parse as urllib

from tempest import config
from tempest.lib import exceptions

from barbican_tempest_plugin.services.key_manager.json import base


CONF = config.CONF


class OrderClient(base.BarbicanTempestClient):

    def __init__(self, *args, secret_client=None, container_client=None,
                 **kwargs):
        """Create a new order client

        secret_client and container_client are optional and will be used
        to queue the respective objects for cleanup when given.
        """
        super().__init__(*args, **kwargs)
        self._order_ids = set()
        self._secret_client = secret_client
        self._container_client = container_client

    def list_orders(self, **kwargs):
        uri = "/v1/orders"
        if kwargs:
            uri += "?%s" % urllib.urlencode(kwargs)

        response, body = self.get(uri)
        self.expected_success(200, response.status)
        return json.loads(body.decode("utf-8"))

    def create_order(self, **kwargs):
        uri = "/v1/orders"

        response, body = self.post(uri, json.dumps(kwargs))
        self.expected_success(202, response.status)
        resp = json.loads(body.decode("utf-8"))
        self._order_ids.add(self.ref_to_uuid(resp['order_ref']))
        return resp

    def get_order(self, order_id):
        uri = "v1/orders/%s" % order_id

        response, body = self.get(uri)
        self.expected_success(200, response.status)
        return json.loads(body.decode("utf-8"))

    def delete_order(self, order_id):
        self._order_ids.discard(order_id)
        uri = "/v1/orders/%s" % order_id

        self._queue_cleanup(order_id)

        response, _ = self.delete(uri)
        self.expected_success(204, response.status)
        return

    def cleanup(self):
        """Attempt to delete all orders created by this client

        If this client was instantiated with secret and/or container
        clients, then we try to queue for cleanup any objects generated
        by the orders.
        """
        cleanup_ids = self._order_ids
        self._order_ids = set()
        for order_id in cleanup_ids:
            self._queue_cleanup(order_id)
            try:
                self.delete_order(order_id)
            except exceptions.NotFound:
                continue

    def _queue_cleanup(self, order_id):
        try:
            order = self.get_order(order_id)
        except exceptions.NotFound:
            pass
        except exceptions.Forbidden:
            pass
        else:
            if (self._secret_client is not None) and \
                    (order.get('secret_ref') is not None):
                self._secret_client.queue_for_cleanup(
                    self.ref_to_uuid(order['secret_ref'])
                )
            if (self._container_client is not None) and \
                    (order.get('container_ref') is not None):
                self._container_client.queue_for_cleanup(
                    self.ref_to_uuid(order['container_ref'])
                )



# Copyright (c) 2017 Johns Hopkins University Applied Physics Laboratory
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from oslo_log import log as logging
from tempest.common import utils
from tempest import config
from tempest.lib.common import api_version_utils
from tempest.lib import decorators

from barbican_tempest_plugin.tests.scenario import barbican_manager

CONF = config.CONF
LOG = logging.getLogger(__name__)


class EphemeralStorageEncryptionTest(barbican_manager.BarbicanScenarioTest):
    min_microversion = '2.1'

    """The test suite for encrypted ephemeral storage

    This test verifies the functionality of encrypted ephemeral storage.
    This test performs the following:
        * Creates an image in Glance
        * Boots an instance from the image
        * Writes to a new file in the instance
    """

    @classmethod
    def skip_checks(cls):
        super(EphemeralStorageEncryptionTest, cls).skip_checks()
        if not CONF.ephemeral_storage_encryption.enabled:
            raise cls.skipException(
                'Ephemeral storage encryption is not supported')
        if not CONF.auth.create_isolated_networks:
            # FIXME(redorobt): remove this skip when system-scope admin
            # issue is fixed.
            raise cls.skipException(
                'Ephemeral storage encryption requires isolated networks')

    @classmethod
    def resource_setup(cls):
        super(EphemeralStorageEncryptionTest, cls).resource_setup()
        cls.request_microversion = (
            api_version_utils.select_request_microversion(
                cls.min_microversion,
                CONF.compute.min_microversion))

    @decorators.idempotent_id('afe720b9-8b35-4a3c-8ff3-15841c2d3148')
    @utils.services('compute', 'image')
    def test_encrypted_ephemeral_lvm_storage(self):
        test_string = 'Once upon a time ...'
        client_test_path = '/tmp/ephemeral_disk_encryption_test'
        img_uuid = self.sign_and_upload_image()
        keypair = self.create_keypair()
        security_group = self.create_security_group()
        instance = self.create_server(
            name='signed_img_server',
            image_id=img_uuid,
            key_name=keypair['name'],
            security_groups=[{'name': security_group['name']}],
            wait_until='ACTIVE')
        instance_ip = self.get_server_ip(instance)
        ssh_client = self.get_remote_client(
            instance_ip,
            private_key=keypair['private_key'],
            server=instance)
        ssh_client.exec_command('echo "%s" > %s' % (test_string,
                                                    client_test_path))
        test_output = ssh_client.exec_command('cat %s' % client_test_path)
        self.assertEqual(str(test_string), str(test_output.rstrip()))



# Copyright 2016 SAP SE
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg

service_option = cfg.BoolOpt("barbican",
                             default=True,
                             help="Whether or not barbican is expected to be "
                                  "available")

key_manager_group = cfg.OptGroup(
    name='key_manager',
    title='Key Manager (Barbican) service options'
)

KeyManagerOpts = [
    cfg.StrOpt('min_microversion',
               default=None,
               help="Lower version of the test target microversion range. "
                    "The format is 'X.Y', where 'X' and 'Y' are int values. "
                    "Tempest selects tests based on the range between "
                    "min_microversion and max_microversion. "
                    "If both values are not specified, Tempest avoids tests "
                    "which require a microversion. Valid values are string "
                    "with format 'X.Y' or string 'latest'"),
    cfg.StrOpt('max_microversion',
               default=None,
               help="Upper version of the test target microversion range. "
                    "The format is 'X.Y', where 'X' and 'Y' are int values. "
                    "Tempest selects tests based on the range between "
                    "min_microversion and max_microversion. "
                    "If both values are not specified, Tempest avoids tests "
                    "which require a microversion. Valid values are string "
                    "with format 'X.Y' or string 'latest'"),
    cfg.StrOpt('region',
               default='regionOne',
               help="The barbican region name to use. If no such region is"
                    "found in the service catalog, the first found one is "
                    "used.")
]

barbican_tempest_group = cfg.OptGroup(
    name='barbican_tempest',
    title='Key Manager (Barbican) service options'
)

BarbicanGroupOpts = [
    cfg.BoolOpt('enable_multiple_secret_stores',
                default=False,
                help="Flag to enable mulitple secret store tests")
]

ephemeral_storage_encryption_group = cfg.OptGroup(
    name="ephemeral_storage_encryption",
    title="Ephemeral storage encryption options")

EphemeralStorageEncryptionGroup = [
    cfg.BoolOpt('enabled',
                default=False,
                help="Does the test environment support ephemeral storage "
                     "encryption?"),
    cfg.StrOpt('cipher',
               default='aes-xts-plain64',
               help="The cipher and mode used to encrypt ephemeral storage. "
                    "AES-XTS is recommended by NIST specifically for disk "
                    "storage, and the name is shorthand for AES encryption "
                    "using the XTS encryption mode. Available ciphers depend "
                    "on kernel support. At the command line, type "
                    "'cryptsetup benchmark' to determine the available "
                    "options (and see benchmark results), or go to "
                    "/proc/crypto."),
    cfg.IntOpt('key_size',
               default=256,
               help="The key size used to encrypt ephemeral storage."),
]

image_signature_verification_group = cfg.OptGroup(
    name="image_signature_verification",
    title="Image Signature Verification Options")

ImageSignatureVerificationGroup = [
    cfg.BoolOpt('enforced',
                default=True,
                help="Does the test environment enforce glance image "
                     "verification?"),
    cfg.BoolOpt('certificate_validation',
                default=True,
                help="Does the test environment enforce image signature"
                     "certificate validation?")
]

EnforceScopeGroup = [
    cfg.BoolOpt('barbican',
                default=False,
                deprecated_group='barbican_rbac_scope_verification',
                deprecated_name='enforce_scope',
                help="Does barbican enforce scope and user "
                     "scope-aware policies?"),
]



# Copyright (c) 2016 Johns Hopkins University Applied Physics Laboratory
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from barbican_tempest_plugin.tests.api import base

from tempest import config
from tempest.lib import decorators


CONF = config.CONF


class QuotasTest(base.BaseKeyManagerTest):
    """Quotas API tests."""

    @decorators.idempotent_id('47ebc42b-0e53-4060-b1a1-55bee2c7c43f')
    def test_get_effective_quota(self):
        body = self.quota_client.get_default_project_quota()
        quotas = body.get('quotas')
        self.assertEqual(-1, quotas.get('secrets'))
        self.assertEqual(-1, quotas.get('cas'))
        self.assertEqual(-1, quotas.get('orders'))
        self.assertEqual(-1, quotas.get('containers'))
        self.assertEqual(-1, quotas.get('consumers'))


class ProjectQuotasTest(base.BaseKeyManagerTest):

    credentials = ['admin', ['service_admin', 'key-manager:service-admin']]

    @classmethod
    def skip_checks(cls):
        super().skip_checks()
        if CONF.enforce_scope.barbican:
            # These tests can't be run with the new RBAC rules because
            # the APIs they're testing require system-scoped credentials
            # instead of the project-scoped credentials used here.
            raise cls.skipException("enforce_scope is enabled for barbican, "
                                    "skipping project quota tests.")

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.quota_client = cls.os_roles_service_admin.secret_v1.QuotaClient()

    @decorators.idempotent_id('07dec492-7f19-4d94-a9d7-28c0643db1bc')
    def test_manage_project_quotas(self):
        # Confirm that there are no quotas
        body = self.quota_client.list_quotas()
        self.assertEqual(0, body.get('total'), body)
        self.assertEqual(0, len(body.get('project_quotas')), body)

        # Create a quota set for the test project
        self.create_project_quota(
            self.quota_client.project_id,
            project_quotas={
                'secrets': 30,
                'orders': 10,
                'containers': 20
            }
        )

        # Verify that the quotas can be found via generic listing.
        body = self.quota_client.list_quotas()
        self.assertEqual(1, body.get('total'), body)
        self.assertEqual(1, len(body.get('project_quotas')), body)
        project_quotas = body.get('project_quotas')[0]
        self.assertEqual(
            self.quota_client.project_id,
            project_quotas.get('project_id'),
            body
        )
        project_quotas = project_quotas.get('project_quotas')
        self.assertEqual(30, project_quotas.get('secrets'), body)
        self.assertEqual(10, project_quotas.get('orders'), body)
        self.assertEqual(20, project_quotas.get('containers'), body)

        # Verify that the quotas can be found via specific listing.
        body = self.quota_client.get_project_quota(
            self.quota_client.project_id
        )
        project_quotas = body.get('project_quotas')
        self.assertEqual(30, project_quotas.get('secrets'), body)
        self.assertEqual(10, project_quotas.get('orders'), body)
        self.assertEqual(20, project_quotas.get('containers'), body)

        # Delete the project quota and confirm that it got deleted
        self.delete_project_quota(self.quota_client.project_id)

        body = self.quota_client.list_quotas()
        self.assertEqual(0, body.get('total'), body)
        self.assertEqual(0, len(body.get('project_quotas')), body)



# Copyright (c) 2016 Johns Hopkins University Applied Physics Laboratory
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import time

from tempest.lib import decorators

from barbican_tempest_plugin.tests.api import base


class OrdersTest(base.BaseKeyManagerTest):
    """Orders API tests."""

    @decorators.idempotent_id('077c1729-1950-4e62-a29c-daba4aa186ad')
    def test_create_list_delete_orders(self):
        # Confirm that there are no orders
        body = self.order_client.list_orders()
        self.assertEqual(0, body.get('total'), body)
        self.assertEqual(0, len(body.get('orders')), body)

        # Create some orders
        body = self.create_order(
            type="key",
            meta={
                "name": "order-key-1",
                "algorithm": "AES",
                "bit_length": 256,
                "mode": "cbc",
                "payload_content_type": "application/octet-stream"
            }
        )
        order_id_1 = base._get_uuid(body.get('order_ref'))
        body = self.create_order(
            type="key",
            meta={
                "name": "order-key-2",
                "algorithm": "AES",
                "bit_length": 256,
                "mode": "cbc",
                "payload_content_type": "application/octet-stream"
            }
        )
        order_id_2 = base._get_uuid(body.get('order_ref'))

        # Verify that the orders can be found via generic listing.
        body = self.order_client.list_orders()
        self.assertEqual(2, body.get('total'), body)
        self.assertEqual(2, len(body.get('orders')), body)

        # Wait max 60 seconds for orders to finish
        for _ in range(12):
            orders = self.order_client.list_orders().get('orders')
            statuses = [o['status'] for o in orders]
            if len(set(statuses)) == 1 and statuses[0] == 'ACTIVE':
                break
            time.sleep(5)

        for order in orders:
            self.assertIn(
                base._get_uuid(order.get('order_ref')),
                [order_id_1, order_id_2],
                body
            )
            self.assertIn(
                'secret_ref',
                order.keys()
            )

        # Verify that the orders can be found via specific listing.
        body = self.order_client.get_order(order_id_1)
        self.assertEqual(
            order_id_1,
            base._get_uuid(body.get('order_ref')),
            body
        )
        self.assertIn('created', body, body)
        self.assertIn('creator_id', body, body)
        self.assertIn('meta', body, body)

        meta = body.get('meta')
        self.assertEqual("order-key-1", meta.get('name'), meta)
        self.assertEqual("AES", meta.get('algorithm'), meta)
        self.assertEqual(256, meta.get('bit_length'), meta)
        self.assertEqual("cbc", meta.get('mode'), meta)
        self.assertEqual(
            "application/octet-stream",
            meta.get('payload_content_type'),
            meta
        )

        self.assertIn('secret_ref', body, body)
        self.assertEqual("ACTIVE", body.get('status'), body)
        self.assertEqual("key", body.get('type'), body)
        self.assertIn('updated', body, body)

        body = self.order_client.get_order(order_id_2)
        self.assertEqual(
            order_id_2,
            base._get_uuid(body.get('order_ref')),
            body
        )
        self.assertIn('created', body, body)
        self.assertIn('creator_id', body, body)
        self.assertIn('meta', body, body)

        meta = body.get('meta')
        self.assertEqual("order-key-2", meta.get('name'), meta)
        self.assertEqual("AES", meta.get('algorithm'), meta)
        self.assertEqual(256, meta.get('bit_length'), meta)
        self.assertEqual("cbc", meta.get('mode'), meta)
        self.assertEqual(
            "application/octet-stream",
            meta.get('payload_content_type'),
            meta
        )

        self.assertIn('secret_ref', body, body)
        self.assertEqual("ACTIVE", body.get('status'), body)
        self.assertEqual("key", body.get('type'), body)
        self.assertIn('updated', body, body)

        # Delete one order and confirm that it got deleted
        self.delete_order(order_id_1)

        body = self.order_client.list_orders()
        self.assertEqual(1, body.get('total'), body)
        self.assertEqual(1, len(body.get('orders')), body)

        order = body.get('orders')[0]
        self.assertEqual(
            order_id_2,
            base._get_uuid(order.get('order_ref')),
            body
        )

        # Leave one order behind to get cleaned up by infra



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import abc

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions

from barbican_tempest_plugin.tests.rbac.v1 import base


CONF = config.CONF


class BarbicanV1RbacContainers:

    @abc.abstractmethod
    def test_list_containers(self):
        """Test list_containers policy

        Testing: GET /v1/containers
        This test must check:
          * whether the persona can list containers
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_create_container(self):
        """Test create_container policy

        Testing: POST /v1/containers
        This test must check:
          * whether the persona can create a new container
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_container(self):
        """Test get_container policy

        Testing: GET /v1/containers/{container-id}
        This test must check:
          * whether the persona can get a container
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_container(self):
        """Test delete_container policy

        Testing: DELETE /v1/containers/{container-id}
        This test must check:
          * whether the persona can delete a container
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_container_acl(self):
        """Test GET /v1/containers/{container-id}/acl

        This test must check:
          * whether the persona can get a containers acl
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_update_container_acl(self):
        """Test PATCH /v1/containers/{container-id}/acl

        This test must check:
          * whether the persona can update an existing containers acl
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_create_container_acl(self):
        """Test PUT /v1/containers/{container-id}/acl

        This test must check:
          * whether the persona can create a containers acl
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_container_acl(self):
        """Test DELETE /v1/containers/{container-id}/acl

        This test must check:
          * whether the persona can delete a containers acl
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_list_container_consumers(self):
        """Test list_container_consumers policy

        Testing: GET /v1/containers/{container-id}/consumers
        This test must check:
          * whether the persona can list a containers consumers
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_create_container_consumer(self):
        """Test create_container_consumer policy

        Testing: POST /v1/containers/{container-id}/consumers
        This test must check:
          * whether the persona can create a consumer of the container
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_container_consumer(self):
        """Test delete_container_consumer policy

        Testing: DELETE /v1/containers/{container-id}/consumers
        This test must check:
          * whether the persona can delete a consumer of the container
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_container_consumer(self):
        """Test GET /v1/containers/{container-id}/consumers/{consumer-id}

        This test must check:
          * whether the persona can get a containers consumer by id

        NOTE: This route is undocumented, also there's no way to get a
        consumer-id back from the API.
        """
        pass

    @abc.abstractmethod
    def test_add_secret_to_container(self):
        """Test add_secret_to_container policy

        Testing: POST /v1/containers/{container-id}/secrets
        This test must check:
          * whether the persona can add a secret to a container
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_secret_from_container(self):
        """Test delete_secret_from_container policy

        Testing: DELETE /v1/containers/{container-id}/secrets
        This test must check:
          * whether the persona can delete a secret from a container
        """
        raise NotImplementedError


class ProjectReaderTests(base.BarbicanV1RbacBase, BarbicanV1RbacContainers):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.secret_v1.ContainerClient()
        cls.consumer_client = cls.os_project_reader.secret_v1.ConsumerClient()

    def setUp(self):
        super().setUp()
        self.secret_id = self.create_test_secret(
            self.secret_client,
            data_utils.rand_name('test-containers'),
            'SECRET_PASSPHRASE'
        )
        self.container_id = self.create_test_container(
            self.container_client,
            data_utils.rand_name('test-containers'))
        self.valid_acl = {
            'read': {
                'users': [self.other_secret_client.user_id],
                'project-access': True
            }
        }
        self.test_consumer = {
            "name": "test-consumer",
            "URL": "https://example.test/consumer"
        }
        self.member_consumer_client.add_consumer_to_container(
            self.container_id,
            **self.test_consumer
        )

    @decorators.idempotent_id('1f0879ae-7e02-4d37-a37c-ee9bc11b40ef')
    def test_list_containers(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.list_containers)

    @decorators.idempotent_id('91406373-339d-4322-bf69-e2842564b9d1')
    def test_create_container(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.create_container)

    @decorators.idempotent_id('c2f5e209-82db-48c7-b11b-2719d3305753')
    def test_get_container(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_container,
            container_id=self.container_id)

    @decorators.idempotent_id('af342d75-67de-4341-9e96-6a58c75a3226')
    def test_delete_container(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.delete_container,
            container_id=self.container_id)

    @decorators.idempotent_id('baa049d5-72a3-4848-8997-a4abd43e00b3')
    def test_get_container_acl(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_container_acl,
            self.container_id)

    @decorators.idempotent_id('6b330e3b-4696-496c-9d65-1cdff39439b9')
    def test_update_container_acl(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.patch_container_acl,
            self.container_id,
            self.valid_acl)

    @decorators.idempotent_id('f78cd73f-e01f-47d3-be5a-83f56837efa1')
    def test_create_container_acl(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.put_container_acl,
            self.container_id,
            self.valid_acl)

    @decorators.idempotent_id('79ef4f66-4a77-44fd-b014-94c075c26bd6')
    def test_delete_container_acl(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.delete_container,
            self.container_id)

    @decorators.idempotent_id('71b335fc-8cb6-4229-bd02-866df4ede926')
    def test_list_container_consumers(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.consumer_client.list_consumers_in_container,
            self.container_id)

    @decorators.idempotent_id('7c64295e-5584-4673-99f8-a87275072f95')
    def test_create_container_consumer(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.consumer_client.add_consumer_to_container,
            self.container_id,
            **self.test_consumer)

    @decorators.idempotent_id('f3dac21e-cfc2-4a97-bd0e-2a207241e9cf')
    def test_delete_container_consumer(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.consumer_client.delete_consumer_from_container,
            self.container_id,
            **self.test_consumer)

    @decorators.idempotent_id('96cfab13-67f6-488b-868d-f0609c26553a')
    def test_add_secret_to_container(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.add_secret_to_container,
            container_id=self.container_id,
            secret_id=self.secret_id)

    @decorators.idempotent_id('d9590a9d-a8be-481e-b50f-22f26973cba4')
    def test_delete_secret_from_container(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.client.delete_secret_from_container,
            container_id=self.container_id,
            secret_id=self.secret_id)


class ProjectMemberTests(ProjectReaderTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.container_client
        cls.consumer_client = cls.member_consumer_client

    @decorators.idempotent_id('028f8858-d463-4f51-befc-63a350e87da2')
    def test_list_containers(self):
        resp = self.client.list_containers()
        containers = resp['containers']

        self.assertGreaterEqual(len(containers), 1)

    @decorators.idempotent_id('db79462a-e6fd-44f5-9d4d-da3c7fda6896')
    def test_create_container(self):
        container_id = self.create_test_container(
            self.client,
            'test-create-container')

        _ = self.container_client.get_container(container_id)

    @decorators.idempotent_id('8d47fec6-487e-43a5-984d-0c3ef08d23e3')
    def test_get_container(self):
        resp = self.client.get_container(self.container_id)

        self.assertEqual(
            self.container_id,
            self.client.ref_to_uuid(resp['container_ref']))

    @decorators.idempotent_id('3c96cec6-02c9-4cae-bea9-be11b22d21aa')
    def test_delete_container(self):
        self.client.delete_container(self.container_id)

        resp = self.container_client.list_containers()
        container_ids = [self.client.ref_to_uuid(c['container_ref'])
                         for c in resp['containers']]
        self.assertNotIn(self.container_id, container_ids)

    @decorators.idempotent_id('64f5bc21-51b5-451a-932b-e6091bf113c3')
    def test_add_secret_to_container(self):
        self.client.add_secret_to_container(
            container_id=self.container_id,
            secret_id=self.secret_id)

        resp = self.client.get_container(self.container_id)
        secret_ids = [self.client.ref_to_uuid(sr['secret_ref'])
                      for sr in resp['secret_refs']]
        self.assertIn(self.secret_id, secret_ids)

    @decorators.idempotent_id('8efb0a35-9b6d-4bde-84f7-43a72ef86bf3')
    def test_delete_secret_from_container(self):
        self.client.add_secret_to_container(
            self.container_id,
            self.secret_id)
        resp = self.client.get_container(self.container_id)
        secret_ids = [self.client.ref_to_uuid(sr['secret_ref'])
                      for sr in resp['secret_refs']]
        self.assertIn(self.secret_id, secret_ids)

        self.client.delete_secret_from_container(
            self.container_id,
            self.secret_id)

        resp = self.client.get_container(self.container_id)
        secret_ids = [self.client.ref_to_uuid(sr['secret_ref'])
                      for sr in resp['secret_refs']]
        self.assertNotIn(self.secret_id, secret_ids)

    @decorators.idempotent_id('167a7fd7-3a08-4d9b-8e5f-2e0d438deea4')
    def test_get_container_acl(self):
        resp = self.client.get_container_acl(self.container_id)
        self.assertIn('read', resp.keys())

    @decorators.idempotent_id('072fd676-36da-44e5-ab53-29a0f1dfe3f1')
    def test_create_container_acl(self):
        _ = self.client.put_container_acl(self.container_id, self.valid_acl)

        acl = self.client.get_container_acl(self.container_id)
        self.assertIn(self.other_secret_client.user_id, acl['read']['users'])

    @decorators.idempotent_id('f5fb9d69-5942-45a3-8c71-2d723e816d52')
    def test_update_container_acl(self):
        _ = self.client.put_container_acl(self.container_id, self.valid_acl)
        acl = self.client.get_container_acl(self.container_id)
        self.assertIn(self.other_secret_client.user_id, acl['read']['users'])
        clear_users_acl = {
            'read': {
                'users': []
            }
        }

        _ = self.client.patch_container_acl(self.container_id, clear_users_acl)

        acl = self.client.get_container_acl(self.container_id)
        self.assertNotIn(self.other_secret_client.user_id,
                         acl['read']['users'])

    @decorators.idempotent_id('03a94e74-13da-40d5-ac63-19e9e7552379')
    def test_delete_container_acl(self):
        _ = self.client.put_container_acl(self.container_id, self.valid_acl)
        acl = self.client.get_container_acl(self.container_id)
        self.assertIn(self.other_secret_client.user_id, acl['read']['users'])

        _ = self.client.delete_container_acl(self.container_id)

        acl = self.client.get_container_acl(self.container_id)
        self.assertNotIn('users', acl['read'].keys())

    @decorators.idempotent_id('7d171230-0ec5-443c-84df-ba79bea52064')
    def test_list_container_consumers(self):
        resp = self.consumer_client.list_consumers_in_container(
            self.container_id
        )
        self.assertEqual(1, resp['total'])

    @decorators.idempotent_id('a7f3b9cb-7be1-49bb-9505-e8b16e30b24f')
    def test_create_container_consumer(self):
        second_consumer = {
            'name': 'another-test-consumer',
            'URL': 'https://exlample.test/consumer/two'
        }

        resp = self.consumer_client.add_consumer_to_container(
            self.container_id,
            **second_consumer)

        self.assertEqual(2, len(resp['consumers']))

    @decorators.idempotent_id('b28e5ed5-797a-46fc-81c4-96a8bfa80757')
    def test_delete_container_consumer(self):
        resp = self.consumer_client.delete_consumer_from_container(
            self.container_id,
            **self.test_consumer)

        self.assertEqual(0, len(resp['consumers']))


class ProjectAdminTests(ProjectMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.admin_container_client
        cls.consumer_client = cls.admin_consumer_client


class ProjectReaderTestsAcrossProjects(ProjectReaderTests):
    """Tests for Project Reader across Projects

    Tests for Project Reader Persona using containers/secrets
    that belong to a different project.

    This class overrides setUp to create self.secret_id and
    self.container_id to use objects that belong to a different
    project.

    We re-use most of the tests in ProjectReaderTests because
    we also expect these to be Forbidden.

    The only exception is the two tests we've overridden to
    pass because it is not possible to list or create containers
    on a different project.
    """

    def setUp(self):
        super().setUp()
        self.secret_id = self.create_test_secret(
            self.other_secret_client,
            data_utils.rand_name('test-containers'),
            'SECRET_PASSPHRASE'
        )
        self.container_id = self.create_test_container(
            self.other_container_client,
            data_utils.rand_name('test-containers'))

    @decorators.idempotent_id('23857de9-c335-4778-b10b-75191990e20c')
    def test_list_containers(self):
        """This is not possible across projects"""
        pass

    @decorators.idempotent_id('cff55c34-fdca-44ab-a136-3b70c75fd8ff')
    def test_create_container(self):
        """This is not possible across projects"""
        pass


class ProjectMemberTestsAcrossProjects(ProjectReaderTestsAcrossProjects):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.container_client


class ProjectAdminTestsAcrossProjects(ProjectMemberTestsAcrossProjects):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.admin_container_client



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import abc

from tempest.lib import decorators
from tempest.lib import exceptions

from barbican_tempest_plugin.tests.rbac.v1 import base


class BarbicanV1RbacQuota:

    @abc.abstractmethod
    def test_get_effective_project_quota(self):
        """Test getting the effective quota information

        Testing: GET /v1/quotas
        This test must check:
          * whether the persona can retrieve the effective quota for
            their project.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_list_project_quotas(self):
        """Test listing all configured project quotas

        Testing: GET /v1/project-quotas
        This test must check:
          * whether the persona can retrieve all modified quotas for
            the entire system.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_custom_quota_for_project(self):
        """Test getting a custom quota for the persona's project

        Testing: GET /v1/project-quotas/{project-id}
        This test must check:
          * whether the persona can retrieve the custom quota for
            the project in the persona's credentials.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_set_new_quota_for_project(self):
        """Test setting a custom quota for the persona's project

        Testing: PUT /v1/project-quotas/{project-id}
        This test must check:
          * whether the persona can create custom quotas for
            the project in the persona's credentials.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_remove_custom_quota_for_project(self):
        """Test removing a custom quota for the persona's project

        Testing: DELETE /v1/project-quotas/{project-id}
        This test must check:
          * whether the persona can delete custom quotas for
            the project in the persona's credentials.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_custom_quota_for_other_project(self):
        """Test getting a custom quota for a different project

        Testing: GET /v1/project-quotas/{project-id}
        This test must check:
          * whether the persona can retrieve the custom quota for
            a project that is different than the project in the
            persona's credentials.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_set_new_quota_for_other_project(self):
        """Test setting a custom quota for a different project

        Testing: PUT /v1/project-quotas/{project-id}
        This test must check:
          * whether the persona can create custom quotas for a
            project that is different than the project in the
            persona's credentials.
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_remove_custom_quota_for_other_project(self):
        """Test removing a custom quota for a different project

        Testing: DELETE /v1/project-quotas/{project-id}
        This test must check:
          * whether the persona can delete custom quotas for a
            project that is different than the project in the
            persona's credentials.
        """
        raise NotImplementedError


class ProjectReaderTests(base.BarbicanV1RbacBase, BarbicanV1RbacQuota):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.secret_v1.QuotaClient()

    @decorators.idempotent_id('936e480a-f603-4d9e-bc77-9a69ef05ac28')
    def test_get_effective_project_quota(self):
        resp = self.client.get_default_project_quota()
        self.assertIn('quotas', resp)

    @decorators.idempotent_id('528907ad-efd8-481e-b57e-7faed7198405')
    def test_list_project_quotas(self):
        self.assertRaises(exceptions.Forbidden, self.client.list_quotas)

    @decorators.idempotent_id('948afb0d-35e8-4a23-880f-b2dc3eebf1bd')
    def test_get_custom_quota_for_project(self):
        project_id = self.client.project_id
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_project_quota,
            project_id)

    @decorators.idempotent_id('c29af3ed-561e-48da-8a1b-81ba19c943bb')
    def test_set_new_quota_for_project(self):
        project_id = self.client.project_id
        self.assertRaises(
            exceptions.Forbidden,
            self.client.create_project_quota,
            project_id,
            project_quotas={
                "secrets": 1000,
                "orders": 1000,
                "containers": 1000
            }
        )

    @decorators.idempotent_id('7382fb20-01f6-4322-9ebc-5bf6038e3e1b')
    def test_remove_custom_quota_for_project(self):
        project_id = self.client.project_id
        self.assertRaises(
            exceptions.Forbidden,
            self.client.delete_project_quota,
            project_id)

    @decorators.idempotent_id('5d062790-6754-4d21-bd0c-08d4a74fa6f3')
    def test_get_custom_quota_for_other_project(self):
        project_id = self.other_secret_client.project_id
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_project_quota,
            project_id)

    @decorators.idempotent_id('26bd25ab-92c2-4437-a742-f301703ce523')
    def test_set_new_quota_for_other_project(self):
        project_id = self.other_secret_client.project_id
        self.assertRaises(
            exceptions.Forbidden,
            self.client.create_project_quota,
            project_id,
            project_quotas={
                "secrets": 1000,
                "orders": 1000,
                "containers": 1000
            }
        )

    @decorators.idempotent_id('7a763152-c64b-42d5-9669-213681327c58')
    def test_remove_custom_quota_for_other_project(self):
        project_id = self.other_secret_client.project_id
        self.assertRaises(
            exceptions.Forbidden,
            self.client.delete_project_quota,
            project_id)


class ProjectMemberTests(ProjectReaderTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_member.secret_v1.QuotaClient()


class ProjectAdminTests(ProjectMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_admin.secret_v1.QuotaClient()

    @decorators.idempotent_id('9099766d-96b3-448d-b400-3b08563d15fc')
    def test_list_project_quotas(self):
        quotas = self.client.list_quotas()
        self.assertIn("project_quotas", quotas)

    @decorators.idempotent_id('dc5125a6-bc94-4426-ba9d-03e2b03c81a8')
    def test_get_custom_quota_for_project(self):
        project_id = self.client.tenant_id
        self.client.create_project_quota(
            project_id,
            project_quotas={
                "secrets": 1000,
                "orders": 1000,
                "containers": 1000
            })
        quota = self.client.get_project_quota(project_id)
        self.assertIn("project_quotas", quota)

    @decorators.idempotent_id('6b169f51-9b17-4d05-aee8-849b94101246')
    def test_set_new_quota_for_project(self):
        project_id = self.client.tenant_id
        self.client.create_project_quota(
            project_id,
            project_quotas={
                "secrets": 1000,
                "orders": 1000,
                "containers": 1000
            })
        quota = self.client.get_project_quota(project_id)
        self.assertIn("project_quotas", quota)

    @decorators.idempotent_id('7a16b9d6-cfdc-4eb7-9e89-b824c612609e')
    def test_remove_custom_quota_for_project(self):
        project_id = self.client.tenant_id
        self.client.create_project_quota(
            project_id,
            project_quotas={
                "secrets": 1000,
                "orders": 1000,
                "containers": 1000
            })
        quota = self.client.get_project_quota(project_id)
        self.assertIn("project_quotas", quota)
        self.client.delete_project_quota(project_id)
        self.assertRaises(
            exceptions.NotFound,
            self.client.get_project_quota,
            project_id)

    @decorators.idempotent_id('17936c5b-5e89-4717-9826-a22243b947cb')
    def test_get_custom_quota_for_other_project(self):
        project_id = self.other_secret_client.tenant_id
        self.client.create_project_quota(
            project_id,
            project_quotas={
                "secrets": 1000,
                "orders": 1000,
                "containers": 1000
            })
        quota = self.client.get_project_quota(project_id)
        self.assertIn("project_quotas", quota)

    @decorators.idempotent_id('d41c97e6-e77d-4bc4-a72d-b068294a0ef6')
    def test_set_new_quota_for_other_project(self):
        project_id = self.other_secret_client.tenant_id
        self.client.create_project_quota(
            project_id,
            project_quotas={
                "secrets": 1000,
                "orders": 1000,
                "containers": 1000
            })
        quota = self.client.get_project_quota(project_id)
        self.assertIn("project_quotas", quota)

    @decorators.idempotent_id('89fb47fd-bf05-4df0-bd47-282417c110b9')
    def test_remove_custom_quota_for_other_project(self):
        project_id = self.other_secret_client.tenant_id
        self.client.create_project_quota(
            project_id,
            project_quotas={
                "secrets": 1000,
                "orders": 1000,
                "containers": 1000
            })
        quota = self.client.get_project_quota(project_id)
        self.assertIn("project_quotas", quota)
        self.client.delete_project_quota(project_id)
        self.assertRaises(
            exceptions.NotFound,
            self.client.get_project_quota,
            project_id)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import abc

from tempest.lib import decorators
from tempest.lib import exceptions

from barbican_tempest_plugin.tests.rbac.v1 import base


class BarbicanV1RbacOrders:

    @abc.abstractmethod
    def test_list_orders(self):
        """Test list_orders policy

        Testing GET /v1/orders
        This test must check:
          * whether persona can list orders
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_create_order(self):
        """Test create_order policy

        Testing POST /v1/orders
        This test must check:
          * whether persona can create orders
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_order(self):
        """Test get_order policy

        Testing GET /v1/orders/{order-id}
        This test must check:
          * whether persona can get order information
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_order(self):
        """Test delete_order policy

        Testing DELETE /v1/orders/{order-id}
        This test must check:
          * whether persona can delete orders
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_get_other_project_order(self):
        """Test get_order policy

        Testing GET /v1/orders/{order-id}
        This test must check:
          * whether persona can get order information
            for an order that belongs to a different
            project
        """
        raise NotImplementedError

    @abc.abstractmethod
    def test_delete_other_project_order(self):
        """Test delete_order policy

        Testing DELETE /v1/orders/{order-id}
        This test must check:
          * whether persona can delete orders
            that belong to a different project
        """
        raise NotImplementedError


class ProjectReaderTests(base.BarbicanV1RbacBase, BarbicanV1RbacOrders):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_reader.secret_v1.OrderClient()

    @decorators.idempotent_id('78c66385-c8ae-44a7-942e-5e1f87072198')
    def test_list_orders(self):
        self.assertRaises(exceptions.Forbidden, self.client.list_orders)

    @decorators.idempotent_id('fa5f861d-a376-437d-ab88-b3eea9a20403')
    def test_create_order(self):
        self.assertRaises(
            exceptions.Forbidden,
            self.create_test_order,
            self.client,
            'create_orders_s'
        )

    @decorators.idempotent_id('39227b64-4d99-42ce-9acb-0fc4df2949ab')
    def test_get_order(self):
        order_id = self.create_test_order(self.order_client, 'test_get_order')
        self.assertRaises(
            exceptions.Forbidden,
            self.client.get_order,
            order_id=order_id)

    @decorators.idempotent_id('ca5ef19c-19f3-45fd-a20d-920c1bb6414c')
    def test_delete_order(self):
        order_id = self.create_test_order(self.order_client,
                                          'test_delete_order')
        self.assertRaises(
            exceptions.Forbidden,
            self.client.delete_order,
            order_id=order_id)

    @decorators.idempotent_id('9689c961-2b91-4d4d-b3f3-4f185e7ae1cc')
    def test_get_other_project_order(self):
        order_id = self.create_test_order(
            self.other_order_client,
            'test_get_other_project_order')
        self.assertRaises(
            exceptions.NotFound,
            self.client.get_order,
            order_id)

    @decorators.idempotent_id('ee9d022c-90d5-427b-b430-b10323270a49')
    def test_delete_other_project_order(self):
        order_id = self.create_test_order(
            self.other_order_client,
            'test_delete_other_project_order')
        self.assertRaises(
            exceptions.NotFound,
            self.client.delete_order,
            order_id)


class ProjectMemberTests(ProjectReaderTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_member.secret_v1.OrderClient()

    @decorators.idempotent_id('789262f2-34fd-46c3-824a-f780d5b5c603')
    def test_list_orders(self):
        _ = self.create_test_order(self.order_client, 'test_list_orders')
        resp = self.client.list_orders()
        self.assertGreaterEqual(len(resp['orders']), 1)

    @decorators.idempotent_id('898154b7-b4e0-44d8-bf84-e87de5d0b48b')
    def test_create_order(self):
        self.create_test_order(self.client, 'create_orders_s')

    @decorators.idempotent_id('798c715d-37fb-4c8b-89c5-e679b016fde7')
    def test_get_order(self):
        order_id = self.create_test_order(self.order_client, 'test_get_order')
        resp = self.client.get_order(order_id)
        self.assertEqual(order_id, self.client.ref_to_uuid(resp['order_ref']))

    @decorators.idempotent_id('fe26f0a1-bcfe-449d-8fd1-ccc4c28f13c2')
    def test_delete_order(self):
        order_id = self.create_test_order(self.order_client,
                                          'test_delete_order')
        self.client.delete_order(order_id)


class ProjectAdminTests(ProjectMemberTests):

    @classmethod
    def setup_clients(cls):
        super().setup_clients()
        cls.client = cls.os_project_admin.secret_v1.OrderClient()



# Copyright 2016 SAP SE
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import json
import urllib.parse

from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions

from barbican_tempest_plugin.services.key_manager.json import base


CONF = config.CONF


class SecretClient(base.BarbicanTempestClient):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self._secret_ids = set()

    def create_secret(self, expected_status=201, **kwargs):
        if 'name' not in kwargs:
            kwargs['name'] = data_utils.rand_name("tempest-sec")

        if 'payload' in kwargs and type(kwargs['payload']) is bytes:
            kwargs['payload'] = kwargs['payload'].decode('utf-8')

        post_body = kwargs
        body = json.dumps(post_body)
        resp, body = self.post("v1/secrets", body)
        self.expected_success(expected_status, resp.status)
        resp = self._parse_resp(body)
        self._secret_ids.add(self.ref_to_uuid(resp['secret_ref']))
        return resp

    def delete_secret(self, secret_id):
        self._secret_ids.discard(secret_id)
        resp, body = self.delete("v1/secrets/%s" % secret_id)
        self.expected_success(204, resp.status)
        return body

    def list_secrets(self, **kwargs):
        uri = "v1/secrets"
        if kwargs is not None:
            uri = '{base}?'.format(base=uri)

            for key in kwargs.keys():
                uri = '{base}&{name}={value}'.format(
                    base=uri,
                    name=key,
                    value=kwargs[key]
                )
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        return self._parse_resp(body)

    def get_secret_metadata(self, secret_id):
        resp, body = self.get("v1/secrets/%s" % secret_id)
        self.expected_success(200, resp.status)
        return self._parse_resp(body)

    def get_secret_payload(self, secret_id, **kwargs):
        """GET /v1/secrets/{secret_id}/payload

        Retrieve the payload.If kwargs are provided they are added
        to the request as query string parameters.
        """
        content_headers = {
            "Accept": "application/octet-stream"
        }
        uri = "v1/secrets/{}/payload".format(secret_id)
        if kwargs:
            uri += '?'
            uri += urllib.parse.urlencode(kwargs)

        resp, body = self.get(uri, headers=content_headers)
        self.expected_success(200, resp.status)
        return self._parse_resp(body)

    def put_secret_payload(self, secret_id, payload):
        content_headers = {
            "Content-Type": "application/octet-stream",
            "Content-Encoding": "base64"
        }
        resp, body = self.put("v1/secrets/%s" % secret_id,
                              payload,
                              headers=content_headers)
        self.expected_success(204, resp.status)
        return body

    def get_secret_acl(self, secret_id):
        headers = {
            'Accept': 'application/json'
        }
        resp, body = self.get('v1/secrets/{}/acl'.format(secret_id),
                              headers=headers)
        self.expected_success(200, resp.status)
        return json.loads(body)

    def put_secret_acl(self, secret_id, acl):
        req_body = json.dumps(acl)
        resp, body = self.put('v1/secrets/{}/acl'.format(secret_id),
                              req_body)
        self.expected_success(200, resp.status)
        return json.loads(body)

    def patch_secret_acl(self, secret_id, acl):
        req_body = json.dumps(acl)
        resp, body = self.patch('v1/secrets/{}/acl'.format(secret_id),
                                req_body)
        self.expected_success(200, resp.status)
        return json.loads(body)

    def delete_secret_acl(self, secret_id):
        resp, body = self.delete('v1/secrets/{}/acl'.format(secret_id))
        self.expected_success(200, resp.status)
        return json.loads(body)

    def queue_for_cleanup(self, secret_id):
        self._secret_ids.add(secret_id)

    def cleanup(self):
        cleanup_ids = self._secret_ids
        self._secret_ids = set()
        for secret_id in cleanup_ids:
            try:
                self.delete_secret(secret_id)
            except exceptions.NotFound:
                pass



# Copyright (c) 2017 Johns Hopkins University Applied Physics Laboratory
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import testtools

from oslo_log import log as logging

from tempest.api.compute import base as compute_base
from tempest.api.image import base

from tempest.common import utils
from tempest.common import waiters

from tempest import config
from tempest import exceptions
from tempest.lib.common import api_version_utils
from tempest.lib import decorators
from tempest.scenario import manager as tempest_manager

from barbican_tempest_plugin.tests.scenario import barbican_manager

CONF = config.CONF
LOG = logging.getLogger(__name__)


class ImageSigningTest(barbican_manager.BarbicanScenarioTest):
    min_microversion = '2.1'

    @classmethod
    def resource_setup(cls):
        super(ImageSigningTest, cls).resource_setup()
        cls.request_microversion = (
            api_version_utils.select_request_microversion(
                cls.min_microversion,
                CONF.compute.min_microversion))

    @decorators.idempotent_id('4343df3c-5553-40ea-8705-0cce73b297a9')
    @utils.services('compute', 'image')
    def test_signed_image_upload_and_boot(self):
        """Test that Nova boots a signed image.

        The test follows these steps:
            * Create an asymmetric keypair
            * Sign an image file with the private key
            * Create a certificate with the public key
            * Store the certificate in Barbican
            * Store the signed image in Glance
            * Boot the signed image
            * Confirm the instance changes state to Active
        """
        img_uuid = self.sign_and_upload_image()

        LOG.debug("Booting server with signed image %s", img_uuid)
        instance = self.create_server(name='signed_img_server',
                                      image_id=img_uuid,
                                      wait_until='ACTIVE')
        self.servers_client.delete_server(instance['id'])

    @decorators.idempotent_id('74f022d6-a6ef-4458-96b7-541deadacf99')
    @utils.services('compute', 'image')
    @testtools.skipUnless(CONF.image_signature_verification.enforced,
                          "Image signature verification is not enforced")
    def test_signed_image_upload_boot_failure(self):
        """Test that Nova refuses to boot an incorrectly signed image.

        If the create_server call succeeds instead of throwing an
        exception, it is likely that signature verification is not
        turned on.  To turn on signature verification, set
        verify_glance_signatures=True in the nova configuration
        file under the [glance] section.

        The test follows these steps:
            * Create an asymmetric keypair
            * Sign an image file with the private key
            * Create a certificate with the public key
            * Store the certificate in Barbican
            * Store the signed image in Glance
            * Modify the signature to be incorrect
            * Attempt to boot the incorrectly signed image
            * Confirm an exception is thrown
        """

        img_uuid = self.sign_and_upload_image()

        LOG.debug("Modifying image signature to be incorrect")
        patch = [dict(replace='/img_signature', value='fake_signature')]
        self.image_client.update_image(image_id=img_uuid, patch=patch)

        self.assertRaisesRegex(exceptions.BuildErrorException,
                               "Signature verification for the image failed",
                               self.create_server,
                               image_id=img_uuid)


class ImageSigningSnapshotTest(barbican_manager.BarbicanScenarioTest,
                               compute_base.BaseV2ComputeTest):

    @classmethod
    def setup_clients(cls):
        super(ImageSigningSnapshotTest, cls).setup_clients()
        cls.client = cls.servers_client

    @decorators.idempotent_id('f0603dfd-8b2c-44e2-8b0f-d65c87aab257')
    @utils.services('compute', 'image')
    def test_signed_image_upload_boot_snapshot(self):
        """Test that Glance can snapshot an instance using a signed image.

        Verify that a snapshot can be taken of an instance booted from a signed
        image and that the resulting snapshot image has had all image signature
        properties dropped from the original image.

        The test follows these steps:
            * Create an asymmetric keypair
            * Sign an image file with the private key
            * Create a certificate with the public key
            * Store the certificate in Barbican
            * Store the signed image in Glance
            * Boot the signed image
            * Confirm the instance changes state to Active
            * Snapshot the running instance
            * Uploading the snapshot and confirm the state moves to ACTIVE
        """
        img_uuid = self.sign_and_upload_image()
        instance = self.create_server(name='signed_img_server_to_snapshot',
                                      image_id=img_uuid,
                                      wait_until='ACTIVE')

        # Snapshot the instance, wait until the snapshot is active
        image = self.create_image_from_server(instance['id'],
                                              wait_until='ACTIVE')

        # Ensure all img_signature image props have been dropped
        signature_props = ['img_signature_hash_method',
                           'img_signature',
                           'img_signature_key_type',
                           'img_signature_certificate_uuid']
        img_meta = self.compute_images_client.list_image_metadata(image['id'])
        self.assertFalse(any(x in img_meta for x in signature_props))

        self.servers_client.delete_server(instance['id'])


class ImageSigningVolumeTest(barbican_manager.BarbicanScenarioTest,
                             tempest_manager.EncryptionScenarioTest,
                             compute_base.BaseV2ComputeTest,
                             base.BaseV2ImageTest):
    """Tests with signed volumes

    The cinder image signature feature is on by default.
    The set of operation is:
        * Create signed volume or create encrypted signed volume
        * Create and upload signed image
        * Create instance
        * Attach signed volume to instance
    """

    min_microversion = '2.1'

    @classmethod
    def skip_checks(cls):
        super(ImageSigningVolumeTest, cls).skip_checks()
        if not CONF.compute_feature_enabled.attach_encrypted_volume:
            raise cls.skipException("Attach encrypted volumes not supported")
        if not CONF.volume_feature_enabled.extend_attached_volume:
            raise cls.skipException("Extend attached volume not supported")
        if not CONF.volume_feature_enabled.extend_attached_encrypted_volume:
            raise cls.skipException("Extend attached"
                                    "encrypted volume not supported")
        if not CONF.service_available.nova:
            raise cls.skipException("Nova service not available")

    def _create_encrypted_signed_volume(self,
                                        encryption_provider,
                                        volume_type,
                                        key_size=256,
                                        cipher='aes-xts-plain64',
                                        control_location='front-end',
                                        imageRef=None):

        """Create an encrypted signed volume"""
        volume_type = self.create_volume_type(name=volume_type)
        self.create_encryption_type(type_id=volume_type['id'],
                                    provider=encryption_provider,
                                    key_size=key_size,
                                    cipher=cipher,
                                    control_location=control_location)
        return self.create_volume(imageRef=imageRef,
                                  volume_type=volume_type['name'])

    def _volume_create(self, volume_type=None, img_uuid=str):
        """Create extended signed volume or signed volume"""

        # Create encrypted extended signed volume
        if volume_type == 'encrypted':
            volume = self._create_encrypted_signed_volume('luks',
                                                          volume_type='luks',
                                                          imageRef=img_uuid)
            LOG.info("Create encrypted volume%s", volume)
            waiters.wait_for_volume_resource_status(
                self.volumes_client, volume['id'], 'available')
            self.assertEqual(volume['encrypted'], True)
            extend_size = volume['size'] * 2
            self.volumes_client.extend_volume(volume_id=volume['id'],
                                              new_size=extend_size)
            LOG.info("Extend volume %s", volume)
            waiters.wait_for_volume_resource_status(
                self.volumes_client,
                volume['id'], 'available')
            resized_volume = self.volumes_client.show_volume(
                volume['id'])['volume']
            self.assertEqual(extend_size, resized_volume['size'])
            return resized_volume

        # Create signed volume
        if img_uuid:
            volume = self.create_volume(imageRef=img_uuid)
            waiters.wait_for_volume_resource_status(
                self.volumes_client, volume['id'], 'available')
            LOG.info("Create volume from signed image %s", volume)
            return volume

    def _create_instance_attach_volume(self, img_uuid, resized_volume):
        """Create instance and attach extended signed volume

        The method follows these steps:
            * Create instance from signed image
            * Confirm the instance changes state to Active
            * Attach encrypted or signed volume to instance
            * Detach volume from instance
            * Delete instance
        """
        # Create instance from signed image
        instance = self.create_server(name='signed_img_server',
                                      image_id=img_uuid,
                                      wait_until='ACTIVE')
        LOG.info("Create instance with signed image %s", instance)
        instance_id = instance['id']

        # Attach volume to instance
        attachment = self.attach_volume(instance, resized_volume)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                attachment['volumeId'],
                                                'in-use')
        LOG.info("Attach volume %s to instance %s", resized_volume, instance)
        self.assertEqual(img_uuid, instance['image']['id'])

        instance_observed = \
            self.servers_client.show_server(instance_id)['server']
        self.assertEqual(
            resized_volume['id'],
            instance_observed['os-extended-volumes:volumes_attached'][0]['id'])

        self.delete_server(instance_observed['id'])

    @decorators.idempotent_id('72ca044d-39a4-4966-b302-f53a446d3e29')
    @decorators.attr(type='slow')
    @utils.services('compute', 'image', 'volume')
    def test_extend_encrypted_signed_volume_attach_to_instance(self):
        """Create volume from signed image, extend volume

        and attach volume to instance.
        Verify that volume can be created from signed image and had
        image signature properties.
        The test follows these steps:
            * Create an asymmetric keypair
            * Sign an image file with the private key
            * Create a certificate with the public key
            * Store the certificate in Barbican
            * Store the signed image in Glance
            * Create encrypted signed volume from image and resize volume
            * Create instance from signed image
            * Confirm the instance changes state to Active
            * Attach encrypted signed volume to instance
            * Detach volume from instance
            * Delete instance
        """
        # Create an encrypted volume and extend volume
        img_uuid = self.sign_and_upload_image()
        resized_volume = self._volume_create(volume_type='encrypted',
                                             img_uuid=img_uuid)
        observed_image = self.images_client.show_image(img_uuid)
        self.assertEqual(
            resized_volume['volume_image_metadata']['signature_verified'],
            'True')
        self.assertEqual(
            resized_volume['volume_image_metadata']
            ['img_signature_certificate_uuid'],
            observed_image['img_signature_certificate_uuid'])
        self._create_instance_attach_volume(img_uuid, resized_volume)

    @decorators.idempotent_id('9f28ce2e-362e-46ec-bf56-aebce9cc49fb')
    @decorators.attr(type='slow')
    @utils.services('compute', 'image', 'volume')
    def test_signed_volume_attach_to_instance(self):
        """Create volume from signed image and attach volume to instance

        Verify that volume can be created from signed image and had
        image signature properties.
        The test follows these steps:
            * Create an asymmetric keypair
            * Sign an image file with the private key
            * Create a certificate with the public key
            * Store the certificate in Barbican
            * Store the signed image in Glance
            * Create signed volume from image
            * Create instance from signed image
            * Confirm the instance changes state to Active
            * Attach signed volume to instance
            * Detach volume from instance
            * Delete instance
        """

        # Create image
        img_uuid = self.sign_and_upload_image()

        # Create volume from signed image
        volume = self._volume_create(img_uuid=img_uuid)
        waiters.wait_for_volume_resource_status(self.volumes_client,
                                                volume['id'], 'available')
        observed_image = self.images_client.show_image(img_uuid)
        self.assertEqual(
            volume['volume_image_metadata']['signature_verified'],
            'True')
        self.assertEqual(
            volume['volume_image_metadata']['img_signature_certificate_uuid'],
            observed_image['img_signature_certificate_uuid'])
        self._create_instance_attach_volume(img_uuid, volume)



# Copyright 2012 OpenStack Foundation
# Copyright 2013 IBM Corp.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log

from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions as lib_exc
from tempest.scenario import manager

CONF = config.CONF

LOG = log.getLogger(__name__)


# we inherit from NetworkScenarioTest since some test cases need access to
# check_*_connectivity methods to validate instances are up and accessible
class ScenarioTest(manager.NetworkScenarioTest):
    """Base class for scenario tests. Uses tempest own clients. """
    # ## Test functions library
    #
    # The create_[resource] functions only return body and discard the
    # resp part which is not used in scenario tests

    def _image_create(self, name, fmt, path,
                      disk_format=None, properties=None):
        if properties is None:
            properties = {}
        name = data_utils.rand_name('%s-' % name)
        params = {
            'name': name,
            'container_format': fmt,
            'disk_format': disk_format or fmt,
            'visibility': 'private'
        }
        # Additional properties are flattened out in the v2 API.
        params.update(properties)
        body = self.image_client.create_image(**params)
        image = body['image'] if 'image' in body else body
        self.addCleanup(self.image_client.delete_image, image['id'])
        self.assertEqual("queued", image['status'])
        with open(path, 'rb') as image_file:
            self.image_client.store_image_file(image['id'], image_file)

        if CONF.image_feature_enabled.import_image:
            available_stores = []
            try:
                available_stores = self.image_client.info_stores()['stores']
            except lib_exc.NotFound:
                pass
            available_import_methods = self.image_client.info_import()[
                'import-methods']['value']
            if ('copy-image' in available_import_methods and
                    len(available_stores) > 1):
                self.image_client.image_import(image['id'],
                                               method='copy-image',
                                               all_stores=True,
                                               all_stores_must_succeed=False)
                failed_stores = waiters.wait_for_image_copied_to_stores(
                    self.image_client, image['id'])
                self.assertEqual(0, len(failed_stores),
                                 "Failed to copy the following stores: %s" %
                                 str(failed_stores))

        return image['id']



# Copyright (c) 2017 Johns Hopkins University Applied Physics Laboratory
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import json
from urllib import parse as urllib

from tempest import config

from barbican_tempest_plugin.services.key_manager.json import base


CONF = config.CONF


class QuotaClient(base.BarbicanTempestClient):

    def list_quotas(self, **kwargs):
        uri = "v1/project-quotas"
        if kwargs:
            uri += "?%s" % urllib.urlencode(kwargs)

        response, body = self.get(uri)
        self.expected_success(200, response.status)
        return json.loads(body.decode("utf-8"))

    def get_default_project_quota(self):
        uri = "v1/quotas"

        response, body = self.get(uri)
        self.expected_success(200, response.status)
        return json.loads(body.decode("utf-8"))

    def get_project_quota(self, project_id):
        uri = "v1/project-quotas/%s" % project_id

        response, body = self.get(uri)
        self.expected_success(200, response.status)
        return json.loads(body.decode("utf-8"))

    def create_project_quota(self, project_id, **kwargs):
        uri = "v1/project-quotas/%s" % project_id

        response, body = self.put(uri, json.dumps(kwargs))
        self.expected_success(204, response.status)
        return

    def delete_project_quota(self, project_id):
        uri = "v1/project-quotas/%s" % project_id

        response, _ = self.delete(uri)
        self.expected_success(204, response.status)
        return



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json
from urllib import parse

from barbican_tempest_plugin.services.key_manager.json import base


class TransportKeyClient(base.BarbicanTempestClient):

    def list_transport_keys(self, **kwargs):
        uri = '/v1/transport_keys'
        if kwargs:
            uri += '?{}'.format(parse.urlencode(kwargs))
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        return json.loads(body.decode('UTF-8'))

    def create_transport_key(self, **kwargs):
        uri = '/v1/transport_keys'
        post_body = json.dumps(kwargs)
        resp, body = self.post(uri, post_body)
        self.expected_success(201, resp.status)
        return json.loads(body.decode('UTF-8'))

    def get_transport_key(self, transport_key_id):
        uri = '/v1/transport_keys/{}'.format(transport_key_id)
        resp, body = self.get(uri)
        self.expected_success(200, resp.status)
        return json.loads(body.decode('UTF-8'))

    def delete_transport_key(self, transport_key_id):
        uri = '/v1/transport_keys/{}'.format(transport_key_id)
        resp, body = self.delete(uri)
        self.expected_success(204, resp.status)
